Package: mx.crypto
Type: Package
Title: Matrix End-to-End Encryption Primitives
Version: 0.2.1
Date: 2026-08-04
Authors@R: c(
    person("Troy", "Hernandez", role = c("aut", "cre"),
           email = "troy@cornball.ai",
           comment = c(ORCID = "0009-0005-4248-604X")),
    person("cornball.ai", role = "cph"),
    person("The Matrix.org Foundation C.I.C.", role = c("ctb", "cph"),
           comment = "Authors of the bundled 'vodozemac' Rust crate; see inst/AUTHORS"),
    person("Authors of the dependency Rust crates", role = "ctb",
           comment = "see inst/AUTHORS for details"))
Description: 'Olm' and 'Megolm' encryption ratchet primitives for the
    'Matrix' messaging protocol <https://matrix.org/>, wrapping the
    'vodozemac' Rust crate. Provides device-key generation, one-time-key
    management, 1:1 'Olm' sessions, and 'Megolm' group sessions. Pairs
    with the 'mx.api' package, which handles 'Matrix' HTTP transport.
License: MIT + file LICENSE | Apache License 2.0
URL: https://github.com/cornball-ai/mx.crypto
BugReports: https://github.com/cornball-ai/mx.crypto/issues
Depends: R (>= 4.3)
SystemRequirements: Cargo (Rust's package manager), rustc (>= 1.85)
Suggests: tinytest, mx.api (>= 0.2.0), simplermarkdown
VignetteBuilder: simplermarkdown
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2026-08-04 20:27:42 UTC; troy
Author: Troy Hernandez [aut, cre] (ORCID:
    <https://orcid.org/0009-0005-4248-604X>),
  cornball.ai [cph],
  The Matrix.org Foundation C.I.C. [ctb, cph] (Authors of the bundled
    'vodozemac' Rust crate; see inst/AUTHORS),
  Authors of the dependency Rust crates [ctb] (see inst/AUTHORS for
    details)
Maintainer: Troy Hernandez <troy@cornball.ai>
Repository: CRAN
Date/Publication: 2026-08-04 22:10:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-08-05 01:15:21 UTC; unix
Archs: mx.crypto.so.dSYM

# tinyrox says don't edit this manually, but it can't stop you!

export(mxc_account_fallback_key)
export(mxc_account_generate_one_time_keys)
export(mxc_account_identity_keys)
export(mxc_account_mark_published)
export(mxc_account_new)
export(mxc_account_one_time_keys)
export(mxc_account_pickle)
export(mxc_account_sign)
export(mxc_account_unpickle)
export(mxc_ed25519_verify)
export(mxc_megolm_decrypt)
export(mxc_megolm_encrypt)
export(mxc_megolm_inbound_new)
export(mxc_megolm_inbound_pickle)
export(mxc_megolm_inbound_unpickle)
export(mxc_megolm_outbound_info)
export(mxc_megolm_outbound_new)
export(mxc_megolm_outbound_pickle)
export(mxc_megolm_outbound_unpickle)
export(mxc_olm_create_inbound)
export(mxc_olm_create_outbound)
export(mxc_olm_decrypt)
export(mxc_olm_encrypt)
export(mxc_olm_session_pickle)
export(mxc_olm_session_unpickle)
export(mxc_verify_device_keys)
export(mxc_verify_one_time_key)

useDynLib(mx.crypto, .registration = TRUE)

# mx.crypto

Olm + Megolm cryptographic ratchet primitives for Matrix, wrapping
[`vodozemac`](https://github.com/matrix-org/vodozemac) (Matrix.org's
pure-Rust crypto crate). Pairs with
[`mx.api`](https://github.com/cornball-ai/mx.api), which handles HTTP
transport.

mx.crypto is crypto only. It does not make HTTP calls, does not
canonicalise JSON, and does not implement cross-signing or SAS
verification (yet).

## Install

Requires a working Rust toolchain (`cargo`, `rustc >= 1.85`).
On Ubuntu: `sudo apt install rustc cargo`. On macOS: `brew install rust`.
Or `rustup` from <https://rustup.rs>.

```r
# GitHub only for now; not on CRAN yet.
remotes::install_github("cornball-ai/mx.crypto")
```

## Quick start: encrypt then decrypt

```r
library(mx.crypto)

# Two devices want to talk
alice <- mxc_account_new()
bob   <- mxc_account_new()

# Bob publishes a one-time key
mxc_account_generate_one_time_keys(bob, 1L)
bob_otks <- mxc_account_one_time_keys(bob)
bob_idk  <- mxc_account_identity_keys(bob)

# Alice opens an Olm session to Bob using one of Bob's OTKs
sess_a <- mxc_olm_create_outbound(
    alice,
    peer_curve25519 = bob_idk$curve25519,
    peer_otk        = bob_otks[[1]]
)

# Alice encrypts; the first message is a pre-key (type 0)
ct <- mxc_olm_encrypt(sess_a, charToRaw("hello"))

# Bob accepts the pre-key, builds the matching inbound session, and
# decrypts in one step
result <- mxc_olm_create_inbound(
    bob,
    peer_curve25519 = mxc_account_identity_keys(alice)$curve25519,
    prekey_b64      = ct$body
)
mxc_account_mark_published(bob)
rawToChar(result$plaintext)
#> [1] "hello"
```

## Verifying homeserver-supplied keys

Real Matrix flows pull device keys and one-time keys from a homeserver
that you cannot fully trust. Verify before you use them:

```r
# After mx.api::mx_keys_query(...) returns a device_keys entry:
keys <- mxc_verify_device_keys(
    device_keys = dk,
    expected_user_id = "@alice:example.org",
    expected_device_id = "ALICEDEV"
)
# `keys` is list(curve25519 = "...", ed25519 = "..."), only present if
# the device's self-signature checked out and its algorithms list
# advertises Olm + Megolm.

# After mx.api::mx_keys_claim(...) returns a signed OTK:
otk <- mxc_verify_one_time_key(
    algorithm_key_id   = "signed_curve25519:AAAA",
    key_object         = signed_otk_obj,
    signing_ed25519    = keys$ed25519,
    expected_user_id   = "@alice:example.org",
    expected_device_id = "ALICEDEV"
)
# `otk` is the verified curve25519 OTK string, ready to feed into
# mxc_olm_create_outbound().
```

Both helpers fail closed: any structural problem, signer mismatch, or
signature-bytes mismatch raises a clear error.
**Identity pinning** (TOFU on first contact + cross-signing) lives
above this layer.

## Status

**0.2.0** (GitHub `main`, 2026-05-13). Not on CRAN yet.

- 0.1.0 was the initial release, GitHub-only.
- 0.2.0 adds `mxc_ed25519_verify` and the two `mxc_verify_*` helpers,
  fixes a latent memory-safety bug in `mxc_olm_create_outbound`
  (`SessionCreationError` is now propagated to R rather than silently
  encoded as a `Session` external pointer), and ships an audit
  vignette + `SECURITY.md`. See `NEWS.md` for the full changelog.

Tested on Ubuntu 24.04 and macOS via the GitHub Actions r-ci workflow,
plus a live homeserver round-trip exercise at
`inst/integration/e2e_demo.R`.

## Security

- `SECURITY.md` (repo root) covers the threat model, the pinned
  vodozemac version, pickle hygiene, and known limitations.
- `vignette("security-audit", package = "mx.crypto")` walks the
  2026-05 audit against the Soatok disclosure, with reproducers and
  fixes.
- Vulnerability reports: email **troy@cornball.ai**.

## License

MIT or Apache 2.0, at your option. The bundled `vodozemac` crate is
Apache 2.0; see `inst/NOTICE` and `inst/AUTHORS` for attribution of all
vendored Rust crates.

# mx.crypto integration scripts

Live network demos that hit a real Matrix homeserver. None of these run
under `R CMD check`, by design.

## e2e_demo.R

Exercises the full round-trip: device-key upload, key query, OTK claim,
Olm handshake, Megolm session sharing via to-device, Megolm-encrypted
room message, and inbound decrypt on the receiver. Uses `mx.api`
(0.1.0.1+) for transport and `mx.crypto` for crypto.

### Requirements

* `mx.api (>= 0.1.0.1)` installed.
* A reachable homeserver. A local Conduit instance is the easiest path
  (`docker run --rm -p 6167:6167 ...`); Synapse works the same way.
* Two accounts on that homeserver who have both joined the same room.

### Environment

| Var             | Meaning                                                       |
|-----------------|---------------------------------------------------------------|
| `MX_HOMESERVER` | Base URL, e.g. `http://localhost:6167`                        |
| `MX_USER_A`     | Localpart for the sender (e.g. `alice`)                       |
| `MX_PASS_A`     | Password for the sender                                       |
| `MX_USER_B`     | Localpart for the receiver (e.g. `bob`)                       |
| `MX_PASS_B`     | Password for the receiver                                     |
| `MX_ROOM_ID`    | Room id both users have joined, e.g. `!xyz:localhost`         |

### Run

```bash
MX_HOMESERVER=http://localhost:6167 \
MX_USER_A=alice MX_PASS_A=alicepass \
MX_USER_B=bob   MX_PASS_B=bobpass \
MX_ROOM_ID='!xyz:localhost' \
Rscript --vanilla "$(Rscript -e 'cat(system.file("integration/e2e_demo.R", package = "mx.crypto"))')"
```

Expected tail of output:

```
B decrypts the Olm pre-key, builds inbound Megolm, decrypts the room
Decrypted room event:
  {"content":{"body":"mx.crypto smoke test ..."},"room_id":"...","type":"m.room.message"}
Done. Test devices logged out.
```

### Notes

* The script creates fresh device ids on each run and uploads new
  identity / one-time keys for them. Existing devices on the same
  accounts are unaffected.
* The room does not need to have `m.room.encryption` set; the script
  emits `m.room.encrypted` events directly. Clients without the
  Megolm session won't decrypt them — that's the point of the demo.
* Once `mx.client` lands, the hand-rolled `curl` PUT for the room send
  will be replaced with `mx.client::mx_send_encrypted()`.

# End-to-end smoke demo: mx.crypto + mx.api against a real homeserver.
#
# Targets a local Conduit (or any Matrix-spec homeserver). Two test
# accounts log in, each holds its own Olm Account; A queries B's keys,
# claims an OTK, opens an Olm session, ships a Megolm session_key over
# to-device, then publishes a Megolm-encrypted room message. B syncs,
# decrypts the to-device Olm pre-key, ingests the session_key, and
# decrypts the room event. Confirms the package primitives plus
# `mx.api 0.1.0.1` transport endpoints are sufficient to wire up E2EE.
#
# This script is intentionally outside the tinytest suite. It hits a
# real network endpoint, mutates server state, and rotates real device
# keys — none of which belongs in `R CMD check`. It is also gated
# behind environment variables so it never runs by accident.
#
# Required env vars:
#   MX_HOMESERVER   base URL, e.g. "http://localhost:6167" for Conduit
#   MX_USER_A       localpart for sender device
#   MX_PASS_A       password for sender
#   MX_USER_B       localpart for receiver device
#   MX_PASS_B       password for receiver
#   MX_ROOM_ID      a room both users have joined
#
# Run with:
#   MX_HOMESERVER=... MX_USER_A=alice MX_PASS_A=... \
#     MX_USER_B=bob MX_PASS_B=... MX_ROOM_ID='!abc:host' \
#     Rscript --vanilla inst/integration/e2e_demo.R

required <- c("MX_HOMESERVER", "MX_USER_A", "MX_PASS_A",
              "MX_USER_B", "MX_PASS_B", "MX_ROOM_ID")
missing <- required[!nzchar(Sys.getenv(required))]
if (length(missing)) {
    stop(sprintf(
        "Missing env vars: %s\nSee header for the full list.",
        paste(missing, collapse = ", ")
    ), call. = FALSE)
}

suppressPackageStartupMessages({
    library(mx.api)
    library(mx.crypto)
})

`%||%` <- function(a, b) if (is.null(a)) b else a

server <- Sys.getenv("MX_HOMESERVER")
room_id <- Sys.getenv("MX_ROOM_ID")

cat("Logging in A and B on", server, "\n")
sA <- mx_login(server, Sys.getenv("MX_USER_A"), Sys.getenv("MX_PASS_A"))
sB <- mx_login(server, Sys.getenv("MX_USER_B"), Sys.getenv("MX_PASS_B"))
cat("  A:", sA$user_id, "/", sA$device_id, "\n")
cat("  B:", sB$user_id, "/", sB$device_id, "\n")

cat("B captures a since-cursor before any traffic flows\n")
syncB0 <- mx_sync(sB, timeout = 0L)
since_B <- syncB0$next_batch

# ---- key publication -----------------------------------------------------

sign_device_keys <- function(account, user_id, device_id) {
    ids <- mxc_account_identity_keys(account)
    unsigned <- list(
        user_id = user_id,
        device_id = device_id,
        algorithms = list("m.olm.v1.curve25519-aes-sha2",
                          "m.megolm.v1.aes-sha2"),
        keys = setNames(
            list(ids$curve25519, ids$ed25519),
            c(paste0("curve25519:", device_id),
              paste0("ed25519:", device_id))
        )
    )
    sig <- mxc_account_sign(account, mx_canonical_json(unsigned))
    out <- unsigned
    out$signatures <- setNames(
        list(setNames(list(sig), paste0("ed25519:", device_id))),
        user_id
    )
    out
}

sign_one_time_keys <- function(account, user_id, device_id, n = 5L) {
    mxc_account_generate_one_time_keys(account, n)
    otks <- mxc_account_one_time_keys(account)
    out <- list()
    for (kid in names(otks)) {
        unsigned <- list(key = otks[[kid]])
        sig <- mxc_account_sign(account, mx_canonical_json(unsigned))
        signed <- unsigned
        signed$signatures <- setNames(
            list(setNames(list(sig), paste0("ed25519:", device_id))),
            user_id
        )
        out[[paste0("signed_curve25519:", kid)]] <- signed
    }
    out
}

acctA <- mxc_account_new()
acctB <- mxc_account_new()
idA <- mxc_account_identity_keys(acctA)
idB <- mxc_account_identity_keys(acctB)

cat("Uploading device + OTKs for A and B\n")
upA <- mx_keys_upload(
    sA,
    device_keys = sign_device_keys(acctA, sA$user_id, sA$device_id),
    one_time_keys = sign_one_time_keys(acctA, sA$user_id, sA$device_id, 5L)
)
mxc_account_mark_published(acctA)

upB <- mx_keys_upload(
    sB,
    device_keys = sign_device_keys(acctB, sB$user_id, sB$device_id),
    one_time_keys = sign_one_time_keys(acctB, sB$user_id, sB$device_id, 5L)
)
mxc_account_mark_published(acctB)

cat("  A OTK counts:",
    paste(names(upA$one_time_key_counts %||% list()),
          unlist(upA$one_time_key_counts %||% list()),
          sep = "=", collapse = " "), "\n")
cat("  B OTK counts:",
    paste(names(upB$one_time_key_counts %||% list()),
          unlist(upB$one_time_key_counts %||% list()),
          sep = "=", collapse = " "), "\n")

# ---- A discovers every device in the room and shares one Megolm key ----

cat("A lists joined room members\n")
members <- mx_room_members(sA, room_id)
cat("  members:", paste(members, collapse = ", "), "\n")

cat("A queries every device for every member\n")
q <- mx_keys_query(
    sA,
    setNames(lapply(members, function(u) character()), members)
)

# Sanity: B's identity matches our local Account
deviceB <- q$device_keys[[sB$user_id]][[sB$device_id]]
stopifnot(!is.null(deviceB))
stopifnot(
    deviceB$keys[[paste0("curve25519:", sB$device_id)]] == idB$curve25519,
    deviceB$keys[[paste0("ed25519:", sB$device_id)]] == idB$ed25519
)

# Collect targets: every (user, device) reported by /keys/query, minus
# A's own device. Devices that haven't published keys (e.g. the live
# cornelius bot if it isn't crypto-aware) won't appear in the response,
# so they're skipped naturally.
targets <- list()
for (uid in names(q$device_keys)) {
    for (did in names(q$device_keys[[uid]])) {
        if (uid == sA$user_id && did == sA$device_id) {
            next
        }
        targets[[length(targets) + 1L]] <- list(
            user_id = uid,
            device_id = did,
            keys = q$device_keys[[uid]][[did]]$keys
        )
    }
}
cat("  share targets:", length(targets), "device(s)\n")
for (t in targets) {
    cat("   -", t$user_id, "/", t$device_id, "\n")
}

cat("A claims one OTK per target (batched)\n")
claim_body <- list()
for (t in targets) {
    if (is.null(claim_body[[t$user_id]])) {
        claim_body[[t$user_id]] <- list()
    }
    claim_body[[t$user_id]][[t$device_id]] <- "signed_curve25519"
}
cl <- if (length(claim_body)) {
    mx_keys_claim(sA, claim_body)
} else {
    list(one_time_keys = list())
}

cat("A creates the outbound Megolm session\n")
megolmA <- mxc_megolm_outbound_new()
info <- mxc_megolm_outbound_info(megolmA)
cat("  session_id:", info$session_id, "\n")

cat("A opens an Olm session + ships m.room_key to each target\n")
td_messages <- list()
shared <- 0L
skipped <- 0L
for (t in targets) {
    otk_obj <- cl$one_time_keys[[t$user_id]][[t$device_id]]
    if (length(otk_obj) == 0L) {
        cat("  no OTK available for", t$user_id, "/", t$device_id,
            "- skipping\n")
        skipped <- skipped + 1L
        next
    }
    otk_value <- otk_obj[[1L]]$key
    peer_curve <- t$keys[[paste0("curve25519:", t$device_id)]]
    peer_ed <- t$keys[[paste0("ed25519:", t$device_id)]]

    olm <- mxc_olm_create_outbound(acctA, peer_curve, otk_value)
    payload <- list(
        type = "m.room_key",
        content = list(
            algorithm = "m.megolm.v1.aes-sha2",
            room_id = room_id,
            session_id = info$session_id,
            session_key = info$session_key
        ),
        sender = sA$user_id,
        recipient = t$user_id,
        recipient_keys = list(ed25519 = peer_ed),
        keys = list(ed25519 = idA$ed25519)
    )
    ct <- mxc_olm_encrypt(olm, charToRaw(mx_canonical_json(payload)))

    if (is.null(td_messages[[t$user_id]])) {
        td_messages[[t$user_id]] <- list()
    }
    td_messages[[t$user_id]][[t$device_id]] <- list(
        algorithm = "m.olm.v1.curve25519-aes-sha2",
        sender_key = idA$curve25519,
        ciphertext = setNames(
            list(list(type = ct$type, body = ct$body)),
            peer_curve
        )
    )
    shared <- shared + 1L
}
cat("  shared to", shared, "device(s),", skipped, "skipped\n")

if (shared > 0L) {
    mx_send_to_device(sA, "m.room.encrypted", td_messages)
}

cat("A posts a Megolm-encrypted m.room.encrypted to the room\n")
room_event_plain <- list(
    type = "m.room.message",
    room_id = room_id,
    content = list(
        msgtype = "m.text",
        body = sprintf("mx.crypto smoke test %s", Sys.time())
    )
)
room_ct <- mxc_megolm_encrypt(
    megolmA, charToRaw(mx_canonical_json(room_event_plain))
)
room_send_path <- sprintf(
    "/_matrix/client/v3/rooms/%s/send/%s/%s",
    utils::URLencode(room_id, reserved = TRUE),
    "m.room.encrypted",
    paste0("mxc-", as.integer(Sys.time()), "-",
           sample.int(.Machine$integer.max, 1))
)
# mx.api has no generic raw-HTTP export, so build the room send via curl
# directly. Once mx.client lands this becomes mx_send_encrypted().
h <- curl::new_handle()
curl::handle_setopt(h, customrequest = "PUT")
curl::handle_setheaders(h, .list = list(
    Authorization = paste("Bearer", sA$token),
    `Content-Type` = "application/json",
    Accept = "application/json"
))
curl::handle_setopt(h, postfields = jsonlite::toJSON(list(
    algorithm = "m.megolm.v1.aes-sha2",
    sender_key = idA$curve25519,
    device_id = sA$device_id,
    session_id = info$session_id,
    ciphertext = room_ct
), auto_unbox = TRUE))
resp <- curl::curl_fetch_memory(
    paste0(sub("/$", "", server), room_send_path), handle = h
)
stopifnot(resp$status_code < 400)

# ---- B decrypts ----------------------------------------------------------

cat("B long-polls /sync for the to-device + room event\n")
collect_enc <- function(sync) {
    rooms <- sync$rooms$join %||% list()
    ev_list <- rooms[[room_id]]$timeline$events %||% list()
    Find(function(e) identical(e$type, "m.room.encrypted") &&
                     identical(e$content$session_id, info$session_id),
         ev_list)
}
collect_td <- function(sync) {
    td <- sync$to_device$events %||% list()
    Find(function(e) identical(e$type, "m.room.encrypted") &&
                     identical(e$content$sender_key, idA$curve25519),
         td)
}

cursor <- since_B
ev_td <- NULL
ev_room <- NULL
for (i in seq_len(4L)) {
    s <- mx_sync(sB, since = cursor, timeout = 5000L)
    if (is.null(ev_td)) {
        ev_td <- collect_td(s)
    }
    if (is.null(ev_room)) {
        ev_room <- collect_enc(s)
    }
    cursor <- s$next_batch
    if (!is.null(ev_td) && !is.null(ev_room)) {
        break
    }
}
stopifnot(!is.null(ev_td), !is.null(ev_room))

ctB <- ev_td$content$ciphertext[[idB$curve25519]]
stopifnot(!is.null(ctB))

cat("B decrypts the Olm pre-key, builds inbound Megolm, decrypts the room\n")
inb <- mxc_olm_create_inbound(acctB, ev_td$content$sender_key, ctB$body)
parsed <- jsonlite::fromJSON(rawToChar(inb$plaintext), simplifyVector = FALSE)
stopifnot(identical(parsed$type, "m.room_key"))
stopifnot(identical(parsed$content$session_id, info$session_id))

igsB <- mxc_megolm_inbound_new(parsed$content$session_key)
dec <- mxc_megolm_decrypt(igsB, ev_room$content$ciphertext)
plain_room <- rawToChar(dec$plaintext)
cat("Decrypted room event:\n  ", plain_room, "\n", sep = "")

try(mx_logout(sA), silent = TRUE)
try(mx_logout(sB), silent = TRUE)
cat("Done. Test devices logged out.\n")

#' Create an outbound Megolm group session
#'
#' The sender side of a Megolm ratchet. Use [mxc_megolm_outbound_info]
#' to read out the `session_key` that must be shared (over Olm) with
#' each recipient device, and [mxc_megolm_encrypt] to encrypt room
#' messages. Rotate (create a new one) on membership changes or after
#' your chosen number-of-messages / time threshold.
#'
#' @return An outbound GroupSession external pointer.
#' @export
mxc_megolm_outbound_new <- function() {
  .Call(.mxc_megolm_outbound_new)
}

#' Inspect an outbound group session
#'
#' Returns the `session_id`, current `session_key` (the value to ship
#' via `m.room_key` to recipient devices), and `message_index`.
#'
#' @param gs An outbound GroupSession.
#' @return Named list: `session_id` (str), `session_key` (str),
#'   `message_index` (int).
#' @export
mxc_megolm_outbound_info <- function(gs) {
  .Call(.mxc_megolm_outbound_info, gs)
}

#' Encrypt a room message
#'
#' @param gs An outbound GroupSession.
#' @param plaintext A `raw` vector. The caller is responsible for
#'   producing the canonical JSON event body.
#' @return Base64 ciphertext (the `ciphertext` field of `m.room.encrypted`).
#' @export
mxc_megolm_encrypt <- function(gs, plaintext) {
  if (!is.raw(plaintext)) {
    stop("'plaintext' must be a raw vector")
  }
  .Call(.mxc_megolm_encrypt, gs, plaintext)
}

#' Pickle an outbound group session
#'
#' @param gs An outbound GroupSession.
#' @param key 32-byte raw vector.
#' @return Base64 string.
#' @export
mxc_megolm_outbound_pickle <- function(gs, key) {
  if (!is.raw(key) || length(key) != 32L) {
    stop("'key' must be a raw vector of length 32")
  }
  .Call(.mxc_megolm_outbound_pickle, gs, key)
}

#' Restore an outbound group session from a pickle
#'
#' @param blob Base64 string.
#' @param key 32-byte raw vector.
#' @return An outbound GroupSession external pointer.
#' @export
mxc_megolm_outbound_unpickle <- function(blob, key) {
  if (!is.raw(key) || length(key) != 32L) {
    stop("'key' must be a raw vector of length 32")
  }
  .Call(.mxc_megolm_outbound_unpickle, as.character(blob), key)
}

#' Build an inbound Megolm session from a shared session_key
#'
#' Constructs the receiver-side ratchet using the `session_key` that
#' was delivered (over Olm) in an `m.room_key` event. The receiver
#' stores the resulting object indexed by `(sender_curve25519,
#' session_id)` and re-uses it to decrypt incoming messages on that
#' session.
#'
#' @param session_key Base64 session_key from `m.room_key`.
#' @return An InboundGroupSession external pointer.
#' @export
mxc_megolm_inbound_new <- function(session_key) {
  .Call(.mxc_megolm_inbound_new, as.character(session_key))
}

#' Decrypt a room message
#'
#' @param igs An InboundGroupSession.
#' @param ciphertext_b64 Base64 ciphertext (the `ciphertext` field of
#'   the `m.room.encrypted` event).
#' @return Named list: `plaintext` (raw) and `message_index` (integer).
#'   Use `message_index` to dedupe replays.
#' @export
mxc_megolm_decrypt <- function(igs, ciphertext_b64) {
  .Call(.mxc_megolm_decrypt, igs, as.character(ciphertext_b64))
}

#' Pickle an inbound group session
#'
#' Pickle after every decrypt so the ratchet state survives restart.
#'
#' @param igs An InboundGroupSession.
#' @param key 32-byte raw vector.
#' @return Base64 string.
#' @export
mxc_megolm_inbound_pickle <- function(igs, key) {
  if (!is.raw(key) || length(key) != 32L) {
    stop("'key' must be a raw vector of length 32")
  }
  .Call(.mxc_megolm_inbound_pickle, igs, key)
}

#' Restore an inbound group session from a pickle
#'
#' @param blob Base64 string.
#' @param key 32-byte raw vector.
#' @return An InboundGroupSession external pointer.
#' @export
mxc_megolm_inbound_unpickle <- function(blob, key) {
  if (!is.raw(key) || length(key) != 32L) {
    stop("'key' must be a raw vector of length 32")
  }
  .Call(.mxc_megolm_inbound_unpickle, as.character(blob), key)
}

library(mx.crypto)

# Outbound (sender) ----------------------------------------------------

gs <- mxc_megolm_outbound_new()
expect_true(is(gs, "externalptr"))

info <- mxc_megolm_outbound_info(gs)
expect_equal(sort(names(info)),
             c("message_index", "session_id", "session_key"))
expect_true(nzchar(info$session_id))
expect_true(nzchar(info$session_key))
expect_equal(info$message_index, 0L)

# Inbound (receiver) ---------------------------------------------------

igs <- mxc_megolm_inbound_new(info$session_key)
expect_true(is(igs, "externalptr"))

# Encrypt / decrypt round-trip ----------------------------------------

ct1_b64 <- mxc_megolm_encrypt(gs, charToRaw("hello room"))
expect_true(is.character(ct1_b64) && nchar(ct1_b64) > 0L)

dec1 <- mxc_megolm_decrypt(igs, ct1_b64)
expect_equal(sort(names(dec1)), c("message_index", "plaintext"))
expect_identical(rawToChar(dec1$plaintext), "hello room")
expect_equal(dec1$message_index, 0L)

# message_index advances
ct2_b64 <- mxc_megolm_encrypt(gs, charToRaw("again"))
dec2 <- mxc_megolm_decrypt(igs, ct2_b64)
expect_identical(rawToChar(dec2$plaintext), "again")
expect_equal(dec2$message_index, 1L)

# outbound info reflects the advance
info2 <- mxc_megolm_outbound_info(gs)
expect_equal(info2$message_index, 2L)
expect_identical(info2$session_id, info$session_id)

# Pickle round-trip ----------------------------------------------------

key <- as.raw(seq_len(32) - 1L)

# outbound
blob_out <- mxc_megolm_outbound_pickle(gs, key)
gs2 <- mxc_megolm_outbound_unpickle(blob_out, key)
ct3_b64 <- mxc_megolm_encrypt(gs2, charToRaw("after pickle"))
dec3 <- mxc_megolm_decrypt(igs, ct3_b64)
expect_identical(rawToChar(dec3$plaintext), "after pickle")

# inbound
blob_in <- mxc_megolm_inbound_pickle(igs, key)
igs2 <- mxc_megolm_inbound_unpickle(blob_in, key)
ct4_b64 <- mxc_megolm_encrypt(gs2, charToRaw("after both pickles"))
dec4 <- mxc_megolm_decrypt(igs2, ct4_b64)
expect_identical(rawToChar(dec4$plaintext), "after both pickles")

# Wrong key length fails up front
expect_error(mxc_megolm_outbound_pickle(gs, raw(31)))
expect_error(mxc_megolm_inbound_pickle(igs, raw(33)))

# Signature verification helpers.
#
# mx.crypto's signing primitive (`mxc_account_sign`) has always existed,
# but until now there was no verification counterpart. That meant
# callers had no way to check `/keys/query` or `/keys/claim` responses
# from the homeserver — a malicious homeserver could substitute its own
# device keys (or its own ed25519 over a real curve25519) and the
# caller would happily open an Olm session into the attacker's hands.
#
# These helpers fail closed: any structural problem or signature
# mismatch raises an error rather than returning a "partially valid"
# object.

#' Verify an Ed25519 signature
#'
#' Thin wrapper over vodozemac's strict Ed25519 verifier
#' (\code{verify_strict}). Returns \code{TRUE} when the signature is
#' valid and \code{FALSE} when it isn't; raises an error if any of
#' \code{public_key}, \code{message}, or \code{signature} is malformed
#' (bad base64, wrong length, etc.).
#'
#' @param public_key Character. Unpadded base64 ed25519 public key
#'   (the same shape \code{mxc_account_identity_keys} returns).
#' @param message A \code{raw} vector. The byte sequence the signer
#'   ran ed25519 over; typically the output of
#'   \code{mx.api::mx_canonical_json()}.
#' @param signature Character. Unpadded base64 ed25519 signature
#'   (86 chars).
#'
#' @return Single logical: \code{TRUE} for a valid signature,
#'   \code{FALSE} otherwise.
#' @examples
#' acct <- mxc_account_new()
#' ids <- mxc_account_identity_keys(acct)
#' sig <- mxc_account_sign(acct, "{\"hello\":\"world\"}")
#' mxc_ed25519_verify(ids$ed25519, charToRaw("{\"hello\":\"world\"}"), sig)
#' @export
mxc_ed25519_verify <- function(public_key, message, signature) {
    if (!is.raw(message)) {
        stop("'message' must be a raw vector", call. = FALSE)
    }
    .Call(.mxc_ed25519_verify,
          as.character(public_key),
          message,
          as.character(signature))
}

#' Verify a Matrix device-keys object
#'
#' Validates a single \code{device_keys} object as returned by
#' \code{/_matrix/client/v3/keys/query}: checks that the structural
#' fields (user_id, device_id, algorithms, keys, signatures) are
#' present and match the expected identity, then verifies the
#' ed25519 self-signature over the canonical-JSON byte sequence
#' (with \code{signatures} and \code{unsigned} stripped).
#'
#' Any of the following raises an error rather than returning
#' \code{FALSE}: missing field, wrong \code{user_id} or
#' \code{device_id}, missing curve25519 / ed25519 key, missing
#' signatures block, signature under the wrong user or device,
#' invalid signature, malformed base64.
#'
#' This function deliberately accepts \emph{any} ed25519 key the
#' object claims for itself; it does not pin against a previously
#' trusted key. Identity pinning is the caller's responsibility
#' (typically TOFU + cross-signing).
#'
#' @param device_keys Named list. The \code{device_keys} object,
#'   exactly as parsed from a homeserver response (no signatures
#'   block stripped).
#' @param expected_user_id Character. Matrix user id the device is
#'   supposed to belong to.
#' @param expected_device_id Character. The device id we're verifying.
#' @param required_algorithms Character vector or NULL. Algorithm
#'   identifiers that must appear in \code{device_keys$algorithms}
#'   for the device to be accepted. \code{NULL} (the default) means
#'   "both Olm and Megolm" — i.e. \code{m.olm.v1.curve25519-aes-sha2}
#'   and \code{m.megolm.v1.aes-sha2}. Pass \code{character(0)} to
#'   skip the check, but be aware that the audit plan requires it.
#'
#' @return Named list with the verified \code{curve25519} and
#'   \code{ed25519} public keys (both base64). Use these as the
#'   inputs for Olm session establishment.
#' @examples
#' \dontrun{
#' q <- mx.api::mx_keys_query(s, list("@alice:server" = "ALICEDEV"))
#' dk <- q$device_keys[["@alice:server"]][["ALICEDEV"]]
#' keys <- mxc_verify_device_keys(dk, "@alice:server", "ALICEDEV")
#' keys$curve25519
#' }
#' @export
mxc_verify_device_keys <- function(device_keys, expected_user_id,
                                   expected_device_id,
                                   required_algorithms = NULL) {
    if (is.null(required_algorithms)) {
        required_algorithms <- c(
            "m.olm.v1.curve25519-aes-sha2",
            "m.megolm.v1.aes-sha2"
        )
    }
    dk <- device_keys
    if (!is.list(dk)) {
        stop("device_keys must be a list", call. = FALSE)
    }
    if (!identical(dk$user_id, expected_user_id)) {
        stop(sprintf(
                     "device_keys user_id mismatch: expected %s, got %s",
                     sQuote(expected_user_id),
                     sQuote(dk$user_id %||% "<missing>")
            ), call. = FALSE)
    }
    if (!identical(dk$device_id, expected_device_id)) {
        stop(sprintf(
                     "device_keys device_id mismatch: expected %s, got %s",
                     sQuote(expected_device_id),
                     sQuote(dk$device_id %||% "<missing>")
            ), call. = FALSE)
    }
    if (length(required_algorithms) > 0L) {
        alg <- dk$algorithms
        if (is.null(alg)) {
            stop("device_keys is missing 'algorithms'", call. = FALSE)
        }
        alg_chr <- tryCatch(
            unlist(alg, use.names = FALSE),
            error = function(e) NULL
        )
        if (!is.character(alg_chr) || length(alg_chr) == 0L) {
            stop("device_keys 'algorithms' must be a non-empty character vector",
                 call. = FALSE)
        }
        missing_alg <- setdiff(required_algorithms, alg_chr)
        if (length(missing_alg) > 0L) {
            stop(sprintf(
                "device_keys 'algorithms' missing required entries: %s",
                paste(sQuote(missing_alg), collapse = ", ")
            ), call. = FALSE)
        }
    }
    if (!is.list(dk$keys) || length(dk$keys) == 0L) {
        stop("device_keys has no 'keys' map", call. = FALSE)
    }
    curve_name <- paste0("curve25519:", expected_device_id)
    ed_name <- paste0("ed25519:", expected_device_id)
    curve <- dk$keys[[curve_name]]
    ed <- dk$keys[[ed_name]]
    if (is.null(curve) || !nzchar(curve)) {
        stop(sprintf("device_keys is missing %s", curve_name), call. = FALSE)
    }
    if (is.null(ed) || !nzchar(ed)) {
        stop(sprintf("device_keys is missing %s", ed_name), call. = FALSE)
    }
    if (!is.list(dk$signatures)) {
        stop("device_keys is unsigned (no signatures block)", call. = FALSE)
    }
    user_sigs <- dk$signatures[[expected_user_id]]
    if (!is.list(user_sigs)) {
        stop(sprintf(
                     "device_keys has no signatures from %s",
                     sQuote(expected_user_id)
            ), call. = FALSE)
    }
    sig <- user_sigs[[ed_name]]
    if (is.null(sig) || !nzchar(sig)) {
        stop(sprintf(
                     "device_keys has no %s signature from %s",
                     ed_name, sQuote(expected_user_id)
            ), call. = FALSE)
    }

    # Verify against the canonical JSON of the device_keys minus
    # signatures + unsigned, per the Matrix signing rule.
    if (!requireNamespace("mx.api", quietly = TRUE)) {
        stop(
             "mxc_verify_device_keys requires mx.api (>= 0.1.0.1) for ",
             "mx_canonical_json()",
             call. = FALSE
        )
    }
    to_sign <- dk
    to_sign$signatures <- NULL
    to_sign$unsigned <- NULL
    msg <- charToRaw(mx.api::mx_canonical_json(to_sign))
    if (!isTRUE(mxc_ed25519_verify(ed, msg, sig))) {
        stop(sprintf(
                     "device_keys signature did not verify for %s / %s",
                     expected_user_id, expected_device_id
            ), call. = FALSE)
    }
    list(curve25519 = curve, ed25519 = ed)
}

#' Verify a signed one-time / fallback key
#'
#' Validates a single signed_curve25519 object returned by
#' \code{/_matrix/client/v3/keys/claim} (or the \code{fallback_keys}
#' block of \code{/keys/upload}). The signing ed25519 key must come
#' from a previously verified \code{device_keys} object —
#' \emph{this function does not look it up for you}, because doing so
#' would silently re-trust whatever the homeserver hands back.
#'
#' The caller must pass the outer map key (\code{"<algorithm>:<key_id>"})
#' alongside the inner key object so the helper can reject anything but
#' \code{signed_curve25519}. The helper also confirms the \code{key}
#' value decodes to a 32-byte curve25519 public key; a signature over
#' garbage bytes is not a useful "verified" result.
#'
#' @param algorithm_key_id Character. The outer map key from the
#'   \code{one_time_keys} or \code{fallback_keys} response, e.g.
#'   \code{"signed_curve25519:AAAAAAAAAAA"}. The algorithm prefix
#'   must be \code{signed_curve25519}.
#' @param key_object Named list. The signed key object (must contain
#'   \code{key} and \code{signatures}).
#' @param signing_ed25519 Character. The base64 ed25519 public key
#'   that should have signed this OTK (i.e. the ed25519 returned by
#'   \code{mxc_verify_device_keys} for the same device).
#' @param expected_user_id Character. Matrix user id the OTK is
#'   supposed to come from.
#' @param expected_device_id Character. Matrix device id.
#'
#' @return Character. The verified curve25519 OTK (base64), ready to
#'   feed into \code{mxc_olm_create_outbound}.
#' @examples
#' \dontrun{
#' cl <- mx.api::mx_keys_claim(s, list(
#'   "@alice:server" = list("ALICEDEV" = "signed_curve25519")
#' ))
#' entry <- cl$one_time_keys[["@alice:server"]][["ALICEDEV"]]
#' algo_kid <- names(entry)[[1]]   # e.g. "signed_curve25519:AAAA"
#' otk <- mxc_verify_one_time_key(
#'   algo_kid, entry[[1]], alice_ed, "@alice:server", "ALICEDEV"
#' )
#' }
#' @export
mxc_verify_one_time_key <- function(algorithm_key_id, key_object,
                                    signing_ed25519, expected_user_id,
                                    expected_device_id) {
    if (!is.character(algorithm_key_id) || length(algorithm_key_id) != 1L ||
        is.na(algorithm_key_id) || !nzchar(algorithm_key_id)) {
        stop("algorithm_key_id must be a single non-empty string ",
             "(e.g. 'signed_curve25519:AAAA')",
             call. = FALSE)
    }
    if (!grepl("^signed_curve25519:.+$", algorithm_key_id)) {
        stop(sprintf(
            "algorithm_key_id %s does not start with 'signed_curve25519:'",
            sQuote(algorithm_key_id)
        ), call. = FALSE)
    }
    if (!is.list(key_object)) {
        stop("key_object must be a list", call. = FALSE)
    }
    key <- key_object$key
    if (is.null(key) || !is.character(key) || length(key) != 1L ||
        is.na(key) || !nzchar(key)) {
        stop("key_object is missing 'key'", call. = FALSE)
    }
    # The 'key' value must be a valid 32-byte curve25519 public key.
    # A signed-but-malformed key would later blow up in
    # mxc_olm_create_outbound; check it here so the helper's promised
    # "ready for session creation" contract holds.
    if (!isTRUE(.Call(.mxc_curve25519_is_valid, as.character(key)))) {
        stop("key_object 'key' is not a valid curve25519 public key",
             call. = FALSE)
    }
    if (!is.list(key_object$signatures)) {
        stop("key_object is unsigned (no signatures block)", call. = FALSE)
    }
    user_sigs <- key_object$signatures[[expected_user_id]]
    if (!is.list(user_sigs)) {
        stop(sprintf(
                     "key_object has no signatures from %s",
                     sQuote(expected_user_id)
            ), call. = FALSE)
    }
    ed_name <- paste0("ed25519:", expected_device_id)
    sig <- user_sigs[[ed_name]]
    if (is.null(sig) || !nzchar(sig)) {
        stop(sprintf(
                     "key_object has no %s signature from %s",
                     ed_name, sQuote(expected_user_id)
            ), call. = FALSE)
    }
    if (!requireNamespace("mx.api", quietly = TRUE)) {
        stop(
             "mxc_verify_one_time_key requires mx.api (>= 0.1.0.1) for ",
             "mx_canonical_json()",
             call. = FALSE
        )
    }
    to_sign <- key_object
    to_sign$signatures <- NULL
    to_sign$unsigned <- NULL
    msg <- charToRaw(mx.api::mx_canonical_json(to_sign))
    if (!isTRUE(mxc_ed25519_verify(signing_ed25519, msg, sig))) {
        stop(sprintf(
                     "one-time key signature did not verify for %s / %s",
                     expected_user_id, expected_device_id
            ), call. = FALSE)
    }
    key
}

`%||%` <- function(a, b) if (is.null(a)) b else a

# Automatically regenerated. Do not edit.

# .Call(.mxc_account_fallback_key, account)
# .Call(.mxc_account_generate_one_time_keys, account, n)
# .Call(.mxc_account_identity_keys, account)
# .Call(.mxc_account_mark_published, account)
# .Call(.mxc_account_new)
# .Call(.mxc_account_one_time_keys, account)
# .Call(.mxc_account_pickle, account, key)
# .Call(.mxc_account_sign, account, canonical_json)
# .Call(.mxc_account_unpickle, blob, key)
# .Call(.mxc_curve25519_is_valid, public_key_b64)
# .Call(.mxc_ed25519_verify, public_key_b64, message, signature_b64)
# .Call(.mxc_megolm_decrypt, igs, ciphertext_b64)
# .Call(.mxc_megolm_encrypt, gs, plaintext)
# .Call(.mxc_megolm_inbound_new, session_key)
# .Call(.mxc_megolm_inbound_pickle, igs, key)
# .Call(.mxc_megolm_inbound_unpickle, blob, key)
# .Call(.mxc_megolm_outbound_info, gs)
# .Call(.mxc_megolm_outbound_new)
# .Call(.mxc_megolm_outbound_pickle, gs, key)
# .Call(.mxc_megolm_outbound_unpickle, blob, key)
# .Call(.mxc_olm_create_inbound, account, peer_curve25519, prekey_b64)
# .Call(.mxc_olm_create_outbound, account, peer_curve25519, peer_otk)
# .Call(.mxc_olm_decrypt, session, message_type, body)
# .Call(.mxc_olm_encrypt, session, plaintext)
# .Call(.mxc_olm_session_pickle, session, key)
# .Call(.mxc_olm_session_unpickle, blob, key)

#' mx.crypto: Matrix End-to-End Encryption Primitives
#'
#' Olm and Megolm ratchet primitives for the Matrix protocol, wrapping
#' the `vodozemac` Rust crate. Pairs with `mx.api`, which handles HTTP
#' transport. mx.crypto is crypto only: no network, no canonical-JSON,
#' no `m.room_key_request`, no cross-signing or SAS in 0.1.0.
#'
#' @name mx.crypto-package
#' @aliases mx.crypto
#' @useDynLib mx.crypto, .registration = TRUE
"_PACKAGE"

#' Create a new Olm Account
#'
#' Creates a fresh device identity. Holds long-lived curve25519 / ed25519
#' identity keys and a one-time-key pool. The returned object is an
#' external pointer; persist it with [mxc_account_pickle].
#'
#' @return An external pointer to a vodozemac Account.
#' @examples
#' acct <- mxc_account_new()
#' mxc_account_identity_keys(acct)
#' @export
mxc_account_new <- function() {
  .Call(.mxc_account_new)
}

#' Public identity keys for an Account
#'
#' Returns the device's two public keys: curve25519 (Olm sender / identity
#' key, used to start sessions) and ed25519 (signing / fingerprint key).
#' Both are unpadded base64 strings, ready for `/keys/upload`.
#'
#' @param account An Account from [mxc_account_new] or [mxc_account_unpickle].
#' @return A named list with `curve25519` and `ed25519` strings.
#' @examples
#' k <- mxc_account_identity_keys(mxc_account_new())
#' k$curve25519
#' @export
mxc_account_identity_keys <- function(account) {
  .Call(.mxc_account_identity_keys, account)
}

#' Sign canonical JSON with the Account's ed25519 key
#'
#' The caller must canonicalise the JSON per the Matrix spec
#' (UTF-8, sorted keys, no whitespace, no insignificant data) before
#' passing it in. mx.crypto does not canonicalise; mx.api will.
#'
#' @param account An Account.
#' @param canonical_json A character string containing canonical JSON.
#' @return Unpadded base64 ed25519 signature.
#' @examples
#' acct <- mxc_account_new()
#' sig <- mxc_account_sign(acct, '{"hello":"world"}')
#' @export
mxc_account_sign <- function(account, canonical_json) {
  .Call(.mxc_account_sign, account, as.character(canonical_json))
}

#' Generate one-time keys
#'
#' Adds `n` curve25519 one-time keys to the Account's pool. Call
#' [mxc_account_one_time_keys] to read them out for upload, then
#' [mxc_account_mark_published] once the homeserver has accepted them.
#'
#' @param account An Account.
#' @param n Number of OTKs to generate.
#' @return Invisible NULL; mutates `account` in place.
#' @examples
#' acct <- mxc_account_new()
#' mxc_account_generate_one_time_keys(acct, 5L)
#' @export
mxc_account_generate_one_time_keys <- function(account, n) {
  invisible(.Call(.mxc_account_generate_one_time_keys, account, as.integer(n)))
}

#' Read pending one-time keys
#'
#' Returns OTKs that have been generated but not yet marked as published.
#' Each value is a curve25519 public key; the caller should sign each
#' with [mxc_account_sign] and upload as `signed_curve25519:<key_id>`.
#'
#' @param account An Account.
#' @return Named list mapping `key_id` to `curve25519_pub` (both
#'   unpadded base64 strings).
#' @examples
#' acct <- mxc_account_new()
#' mxc_account_generate_one_time_keys(acct, 5L)
#' otks <- mxc_account_one_time_keys(acct)
#' @export
mxc_account_one_time_keys <- function(account) {
  .Call(.mxc_account_one_time_keys, account)
}

#' Mark current one-time keys as published
#'
#' Call after the homeserver has accepted a `/keys/upload` containing
#' the keys returned by [mxc_account_one_time_keys]. Future calls to
#' that function will not re-include them.
#'
#' @param account An Account.
#' @return Invisible NULL; mutates `account`.
#' @export
mxc_account_mark_published <- function(account) {
  invisible(.Call(.mxc_account_mark_published, account))
}

#' Generate and return a fallback key
#'
#' Generates a fallback curve25519 key, which the homeserver hands out
#' when an OTK pool is exhausted. Returns the freshly generated key.
#' Calling it again rotates the previous fallback.
#'
#' @param account An Account.
#' @return Named list with `key_id` and `curve25519` (both base64).
#' @export
mxc_account_fallback_key <- function(account) {
  .Call(.mxc_account_fallback_key, account)
}

#' Pickle an Account to an encrypted blob
#'
#' Serialises the Account's state and encrypts it under a 32-byte key.
#' Restore with [mxc_account_unpickle]. Pickle after every state change
#' that you want to survive a restart (OTK generation, fallback key
#' rotation, mark-published).
#'
#' @param account An Account.
#' @param key A `raw` vector of length 32. The caller is responsible for
#'   key derivation; use a KDF on a passphrase, do not pass a passphrase.
#' @return Base64 string containing the encrypted pickle.
#' @export
mxc_account_pickle <- function(account, key) {
  if (!is.raw(key) || length(key) != 32L) {
    stop("'key' must be a raw vector of length 32")
  }
  .Call(.mxc_account_pickle, account, key)
}

#' Restore an Account from an encrypted pickle
#'
#' @param blob A base64 string produced by [mxc_account_pickle].
#' @param key The 32-byte key the pickle was encrypted under.
#' @return An Account external pointer.
#' @export
mxc_account_unpickle <- function(blob, key) {
  if (!is.raw(key) || length(key) != 32L) {
    stop("'key' must be a raw vector of length 32")
  }
  .Call(.mxc_account_unpickle, as.character(blob), key)
}

library(mx.crypto)

# ============================================================================
# mxc_ed25519_verify — round-trip + tamper
# ============================================================================

acct <- mxc_account_new()
ids <- mxc_account_identity_keys(acct)
msg <- charToRaw("{\"hello\":\"world\"}")
sig <- mxc_account_sign(acct, rawToChar(msg))

# Valid round-trip
expect_true(mxc_ed25519_verify(ids$ed25519, msg, sig))

# Tampered message
expect_false(mxc_ed25519_verify(
  ids$ed25519, charToRaw("{\"hello\":\"WORLD\"}"), sig
))

# Tampered signature (last 6 chars rewritten)
bad_sig <- paste0(substr(sig, 1, 80), "AAAAAA")
expect_false(mxc_ed25519_verify(ids$ed25519, msg, bad_sig))

# Wrong public key
other_acct <- mxc_account_new()
other_ed <- mxc_account_identity_keys(other_acct)$ed25519
expect_false(mxc_ed25519_verify(other_ed, msg, sig))

# Malformed public key / signature both raise errors
expect_error(mxc_ed25519_verify("not-base64!", msg, sig))
expect_error(mxc_ed25519_verify(ids$ed25519, msg, "not-base64!"))
# Wrong-length public key (32 chars instead of 43)
expect_error(mxc_ed25519_verify(strrep("A", 32), msg, sig))
# message must be raw
expect_error(mxc_ed25519_verify(ids$ed25519, "not raw", sig))

# ============================================================================
# Regression: mxc_olm_create_outbound rejects non-contributory DH keys
# ============================================================================

# All-zero curve25519: vodozemac returns Err(NonContributoryKey); mx.crypto
# now propagates that as an R error instead of returning a corrupt
# externalptr.
# 32 zero bytes, base64-encoded. Hardcoded so the test doesn't need
# jsonlite as an unconditional dependency.
zero_curve <- "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA="
expect_error(
  mxc_olm_create_outbound(acct, zero_curve, zero_curve),
  pattern = "non-contributory|create_outbound_session"
)

# ============================================================================
# mxc_verify_device_keys — happy path + every rejection branch
# ============================================================================

# Build a real, signed device_keys object the way mx.crypto/mx.api wants.
build_dk <- function(account, user_id, device_id) {
  k <- mxc_account_identity_keys(account)
  unsigned <- list(
    user_id = user_id,
    device_id = device_id,
    algorithms = list("m.olm.v1.curve25519-aes-sha2",
                      "m.megolm.v1.aes-sha2"),
    keys = setNames(
      list(k$curve25519, k$ed25519),
      c(paste0("curve25519:", device_id),
        paste0("ed25519:", device_id))
    )
  )
  s <- mxc_account_sign(account, mx.api::mx_canonical_json(unsigned))
  out <- unsigned
  out$signatures <- setNames(
    list(setNames(list(s), paste0("ed25519:", device_id))),
    user_id
  )
  out
}

if (requireNamespace("mx.api", quietly = TRUE) &&
    utils::packageVersion("mx.api") >= "0.2.0") {
  alice <- mxc_account_new()
  alice_ids <- mxc_account_identity_keys(alice)
  dk_ok <- build_dk(alice, "@alice:example.org", "ALICEDEV")

  # Happy path — returns the verified keys
  keys <- mxc_verify_device_keys(dk_ok, "@alice:example.org", "ALICEDEV")
  expect_equal(keys$curve25519, alice_ids$curve25519)
  expect_equal(keys$ed25519, alice_ids$ed25519)

  # Wrong expected user id
  expect_error(
    mxc_verify_device_keys(dk_ok, "@mallory:example.org", "ALICEDEV"),
    pattern = "user_id mismatch"
  )

  # Wrong expected device id
  expect_error(
    mxc_verify_device_keys(dk_ok, "@alice:example.org", "OTHERDEV"),
    pattern = "device_id mismatch"
  )

  # Missing curve25519
  bad <- dk_ok
  bad$keys[[paste0("curve25519:", "ALICEDEV")]] <- NULL
  expect_error(
    mxc_verify_device_keys(bad, "@alice:example.org", "ALICEDEV"),
    pattern = "missing curve25519"
  )

  # Missing ed25519
  bad <- dk_ok
  bad$keys[[paste0("ed25519:", "ALICEDEV")]] <- NULL
  expect_error(
    mxc_verify_device_keys(bad, "@alice:example.org", "ALICEDEV"),
    pattern = "missing ed25519"
  )

  # No signatures block
  bad <- dk_ok
  bad$signatures <- NULL
  expect_error(
    mxc_verify_device_keys(bad, "@alice:example.org", "ALICEDEV"),
    pattern = "unsigned"
  )

  # Signature under the wrong user id (homeserver re-signs alice's keys
  # as if they were mallory's)
  mallory <- mxc_account_new()
  mallory_sig <- mxc_account_sign(
    mallory,
    mx.api::mx_canonical_json(within(dk_ok, signatures <- NULL))
  )
  bad <- dk_ok
  bad$signatures <- setNames(
    list(setNames(list(mallory_sig), paste0("ed25519:", "ALICEDEV"))),
    "@mallory:example.org"
  )
  expect_error(
    mxc_verify_device_keys(bad, "@alice:example.org", "ALICEDEV"),
    pattern = "no signatures from"
  )

  # Mutated keys content — signature no longer matches
  bad <- dk_ok
  bad$keys[[paste0("curve25519:", "ALICEDEV")]] <- mxc_account_identity_keys(
    mxc_account_new()
  )$curve25519
  expect_error(
    mxc_verify_device_keys(bad, "@alice:example.org", "ALICEDEV"),
    pattern = "did not verify"
  )

  # Signature attached under wrong device id (ed25519:OTHERDEV instead of
  # ed25519:ALICEDEV)
  bad <- dk_ok
  bad$signatures[["@alice:example.org"]] <- setNames(
    list(bad$signatures[["@alice:example.org"]][[paste0("ed25519:",
                                                        "ALICEDEV")]]),
    paste0("ed25519:", "OTHERDEV")
  )
  expect_error(
    mxc_verify_device_keys(bad, "@alice:example.org", "ALICEDEV"),
    pattern = "no ed25519:ALICEDEV signature"
  )

  # algorithms: missing entirely
  bad <- dk_ok
  bad$algorithms <- NULL
  expect_error(
    mxc_verify_device_keys(bad, "@alice:example.org", "ALICEDEV"),
    pattern = "missing 'algorithms'"
  )

  # algorithms: present but missing the required Olm algorithm
  bad <- dk_ok
  bad$algorithms <- list("m.megolm.v1.aes-sha2")  # no olm
  expect_error(
    mxc_verify_device_keys(bad, "@alice:example.org", "ALICEDEV"),
    pattern = "missing required entries.*m\\.olm"
  )

  # algorithms: empty list
  bad <- dk_ok
  bad$algorithms <- list()
  expect_error(
    mxc_verify_device_keys(bad, "@alice:example.org", "ALICEDEV"),
    pattern = "non-empty"
  )

  # algorithms: opt out via required_algorithms = character(0) still passes
  bad <- dk_ok
  bad$algorithms <- NULL
  expect_silent(mxc_verify_device_keys(
    dk_ok, "@alice:example.org", "ALICEDEV",
    required_algorithms = character(0)
  ))

  # ==========================================================================
  # mxc_verify_one_time_key — happy path + every rejection branch
  # ==========================================================================

  mxc_account_generate_one_time_keys(alice, 1L)
  otks <- mxc_account_one_time_keys(alice)
  otk_kid <- names(otks)[[1L]]
  otk_key <- otks[[1L]]
  unsigned_otk <- list(key = otk_key)
  s <- mxc_account_sign(alice, mx.api::mx_canonical_json(unsigned_otk))
  otk_ok <- list(
    key = otk_key,
    signatures = setNames(
      list(setNames(list(s), paste0("ed25519:", "ALICEDEV"))),
      "@alice:example.org"
    )
  )
  algo_kid_ok <- paste0("signed_curve25519:", otk_kid)

  # Happy path
  verified <- mxc_verify_one_time_key(
    algo_kid_ok, otk_ok, alice_ids$ed25519,
    "@alice:example.org", "ALICEDEV"
  )
  expect_equal(verified, otk_key)

  # Wrong signing key (mallory's ed25519 instead of alice's)
  mallory_ids <- mxc_account_identity_keys(mallory)
  expect_error(
    mxc_verify_one_time_key(
      algo_kid_ok, otk_ok, mallory_ids$ed25519,
      "@alice:example.org", "ALICEDEV"
    ),
    pattern = "did not verify"
  )

  # Missing key field
  bad <- otk_ok
  bad$key <- NULL
  expect_error(
    mxc_verify_one_time_key(
      algo_kid_ok, bad, alice_ids$ed25519,
      "@alice:example.org", "ALICEDEV"
    ),
    pattern = "missing 'key'"
  )

  # Unsigned
  bad <- otk_ok
  bad$signatures <- NULL
  expect_error(
    mxc_verify_one_time_key(
      algo_kid_ok, bad, alice_ids$ed25519,
      "@alice:example.org", "ALICEDEV"
    ),
    pattern = "unsigned"
  )

  # Wrong user signed it
  bad <- otk_ok
  bad$signatures <- setNames(bad$signatures, "@mallory:example.org")
  expect_error(
    mxc_verify_one_time_key(
      algo_kid_ok, bad, alice_ids$ed25519,
      "@alice:example.org", "ALICEDEV"
    ),
    pattern = "no signatures from"
  )

  # Mutated key — signature mismatch
  bad <- otk_ok
  bad$key <- mxc_account_identity_keys(mxc_account_new())$curve25519
  expect_error(
    mxc_verify_one_time_key(
      algo_kid_ok, bad, alice_ids$ed25519,
      "@alice:example.org", "ALICEDEV"
    ),
    pattern = "did not verify"
  )

  # Wrong algorithm prefix on the outer map key
  expect_error(
    mxc_verify_one_time_key(
      paste0("curve25519:", otk_kid), otk_ok, alice_ids$ed25519,
      "@alice:example.org", "ALICEDEV"
    ),
    pattern = "does not start with"
  )
  expect_error(
    mxc_verify_one_time_key(
      paste0("signed_ed25519:", otk_kid), otk_ok, alice_ids$ed25519,
      "@alice:example.org", "ALICEDEV"
    ),
    pattern = "does not start with"
  )
  # Empty / NA algorithm_key_id
  expect_error(
    mxc_verify_one_time_key("", otk_ok, alice_ids$ed25519,
                            "@alice:example.org", "ALICEDEV"),
    pattern = "non-empty"
  )
  expect_error(
    mxc_verify_one_time_key(NA_character_, otk_ok, alice_ids$ed25519,
                            "@alice:example.org", "ALICEDEV"),
    pattern = "non-empty"
  )

  # 'key' value that is signed but not a valid curve25519 public key.
  # Build a fresh otk_object where the key is garbage bytes, signed
  # correctly by alice. The signature is over the canonical_json of
  # {"key": "<garbage>"}, so it verifies — but the curve25519 check
  # must still reject before mxc_olm_create_outbound ever sees it.
  bogus_key <- "AAAA"  # short base64, not 32 bytes
  bogus_unsigned <- list(key = bogus_key)
  bogus_sig <- mxc_account_sign(alice,
                                mx.api::mx_canonical_json(bogus_unsigned))
  bogus_otk <- list(
    key = bogus_key,
    signatures = setNames(
      list(setNames(list(bogus_sig), paste0("ed25519:", "ALICEDEV"))),
      "@alice:example.org"
    )
  )
  expect_error(
    mxc_verify_one_time_key(
      paste0("signed_curve25519:invalid"), bogus_otk,
      alice_ids$ed25519, "@alice:example.org", "ALICEDEV"
    ),
    pattern = "valid curve25519 public key"
  )
}

library(mx.crypto)

# --- new account ---------------------------------------------------------

acct <- mxc_account_new()
expect_true(is(acct, "externalptr"))

ids <- mxc_account_identity_keys(acct)
expect_equal(sort(names(ids)), c("curve25519", "ed25519"))
expect_true(is.character(ids$curve25519) && nzchar(ids$curve25519))
expect_true(is.character(ids$ed25519) && nzchar(ids$ed25519))
# unpadded base64 of a 32-byte key is 43 chars
expect_equal(nchar(ids$curve25519), 43L)
expect_equal(nchar(ids$ed25519), 43L)

# --- one-time keys -------------------------------------------------------

# Empty pool initially
expect_equal(length(mxc_account_one_time_keys(acct)), 0L)

mxc_account_generate_one_time_keys(acct, 3L)
otks <- mxc_account_one_time_keys(acct)
expect_equal(length(otks), 3L)
expect_true(all(nzchar(names(otks))))
expect_true(all(vapply(otks, function(k) nchar(k) == 43L, logical(1))))

mxc_account_mark_published(acct)
expect_equal(length(mxc_account_one_time_keys(acct)), 0L)

# --- fallback key --------------------------------------------------------

fk <- mxc_account_fallback_key(acct)
expect_equal(sort(names(fk)), c("curve25519", "key_id"))
expect_true(nzchar(fk$key_id))
expect_equal(nchar(fk$curve25519), 43L)

# --- signing -------------------------------------------------------------

sig <- mxc_account_sign(acct, '{"hello":"world"}')
expect_true(is.character(sig) && nchar(sig) > 0L)
# ed25519 signature unpadded base64 is 86 chars
expect_equal(nchar(sig), 86L)

# --- pickle round-trip ---------------------------------------------------

key <- as.raw(seq_len(32) - 1L) # 0x00..0x1f
blob <- mxc_account_pickle(acct, key)
expect_true(is.character(blob) && nchar(blob) > 0L)

acct2 <- mxc_account_unpickle(blob, key)
ids2 <- mxc_account_identity_keys(acct2)
expect_identical(ids$curve25519, ids2$curve25519)
expect_identical(ids$ed25519, ids2$ed25519)

# Wrong key fails
bad_key <- as.raw(rep(0xff, 32))
expect_error(mxc_account_unpickle(blob, bad_key))

# Wrong key length fails up front
expect_error(mxc_account_pickle(acct, raw(31)))
expect_error(mxc_account_pickle(acct, "not raw"))

#!/usr/bin/env Rscript

fail <- function(...) {
  cat(..., "\n", sep = "")
  quit(save = "no", status = 1L, runLast = FALSE)
}

note <- function(...) {
  cat(..., "\n", sep = "")
}

cargo_bin_dir <- function() {
  sysname <- Sys.info()[["sysname"]]
  if (identical(sysname, "Windows")) {
    userprofile <- Sys.getenv("USERPROFILE", unset = "")
    if (!nzchar(userprofile)) {
      return("")
    }
    return(file.path(userprofile, ".cargo", "bin"))
  }
  path.expand("~/.cargo/bin")
}

tool_executable_name <- function(tool) {
  if (identical(Sys.info()[["sysname"]], "Windows")) {
    paste0(tool, ".exe")
  } else {
    tool
  }
}

tool_path_in_dir <- function(dir, tool) {
  if (!nzchar(dir) || !dir.exists(dir)) {
    return("")
  }
  with_ext <- file.path(dir, tool_executable_name(tool))
  if (file.exists(with_ext)) {
    return(with_ext)
  }
  without_ext <- file.path(dir, tool)
  if (file.exists(without_ext)) {
    return(without_ext)
  }
  ""
}

normalize_tool_path <- function(path) {
  expanded <- path.expand(path)
  if (!file.exists(expanded)) {
    stop(sprintf("Path does not exist: %s", path))
  }
  gsub("\\\\", "/", expanded)
}

extract_semver <- function(version_lines, tool) {
  pattern <- sprintf("^\\s*%s\\s+([0-9]+\\.[0-9]+(\\.[0-9]+)?)\\b", tool)
  for (line in version_lines) {
    m <- regexec(pattern, line)
    parts <- regmatches(line, m)[[1]]
    if (length(parts) >= 2L) {
      return(list(version = parts[[2]], line = line))
    }
  }
  NULL
}

tool_info <- function(tool, tool_path) {
  if (!nzchar(tool_path)) {
    stop(sprintf("Could not find '%s'.", tool))
  }
  tool_path <- normalize_tool_path(tool_path)
  output <- tryCatch(
    system2(tool_path, "--version", stdout = TRUE, stderr = TRUE),
    error = function(e) e
  )
  if (inherits(output, "error")) {
    stop(sprintf(
      "Could not run '%s --version': %s",
      tool_path,
      conditionMessage(output)
    ))
  }
  status <- attr(output, "status")
  if (!is.null(status) && status != 0L) {
    stop(sprintf(
      "'%s --version' returned non-zero exit status: %s",
      tool_path,
      status
    ))
  }
  if (length(output) < 1L || !any(nzchar(output))) {
    stop(sprintf("'%s --version' did not report a version string.", tool_path))
  }
  parsed <- extract_semver(output, tool)
  if (is.null(parsed)) {
    stop(
      sprintf(
        "Could not parse semantic version from '%s --version' output: %s",
        tool_path,
        paste(output, collapse = " | ")
      )
    )
  }
  list(path = tool_path, line = parsed$line, version = parsed$version)
}

read_msrv <- function(cargo_toml_path) {
  lines <- readLines(cargo_toml_path, warn = FALSE)
  for (line in lines) {
    if (grepl("^\\s*#", line)) {
      next
    }
    m <- regexec("^\\s*rust-version\\s*=\\s*\"([^\"]+)\"", line)
    parts <- regmatches(line, m)[[1]]
    if (length(parts) >= 2L) {
      return(parts[[2]])
    }
  }
  fail(sprintf("Could not find 'rust-version' in %s.", cargo_toml_path))
}

extract_vendor_if_present <- function(rust_dir) {
  tarball <- file.path(rust_dir, "vendor.tar.xz")
  if (!file.exists(tarball)) {
    note("No src/rust/vendor.tar.xz found; using default cargo behavior.")
    return(invisible(FALSE))
  }

  note(sprintf("Found %s; extracting vendored crates.", tarball))
  unlink(file.path(rust_dir, "vendor"), recursive = TRUE, force = TRUE)
  unlink(file.path(rust_dir, ".cargo"), recursive = TRUE, force = TRUE)
  unlink(file.path(rust_dir, ".cargo-home"), recursive = TRUE, force = TRUE)

  tar_result <- tryCatch(
    utils::untar(tarball, exdir = rust_dir, tar = "internal"),
    error = function(e) e
  )
  if (inherits(tar_result, "error")) {
    fail(sprintf(
      "Failed to extract %s: %s",
      tarball,
      conditionMessage(tar_result)
    ))
  }
  if (is.numeric(tar_result) && length(tar_result) == 1L && tar_result != 0L) {
    fail(sprintf("Extracting %s failed with status %s.", tarball, tar_result))
  }
  if (!dir.exists(file.path(rust_dir, "vendor"))) {
    fail(sprintf("Expected %s/vendor after extraction.", rust_dir))
  }
  config_path <- file.path(rust_dir, ".cargo", "config.toml")
  if (!file.exists(config_path)) {
    fail(sprintf("Expected %s after extraction.", config_path))
  }
  note("Vendored crates and cargo config are ready.")
  invisible(TRUE)
}

make_toolchain_candidates <- function() {
  path_cargo <- Sys.which("cargo")
  path_rustc <- Sys.which("rustc")
  personal_dir <- cargo_bin_dir()
  personal_cargo <- tool_path_in_dir(personal_dir, "cargo")
  personal_rustc <- tool_path_in_dir(personal_dir, "rustc")

  candidates <- list(
    list(
      name = "system PATH",
      cargo_path = path_cargo,
      rustc_path = path_rustc
    ),
    list(
      name = sprintf("personal cargo bin (%s)", personal_dir),
      cargo_path = personal_cargo,
      rustc_path = personal_rustc
    )
  )

  keep <- rep(TRUE, length(candidates))
  if (!nzchar(personal_dir)) {
    keep[[2]] <- FALSE
  }
  candidates <- candidates[keep]

  deduped <- list()
  seen <- character()
  for (candidate in candidates) {
    key <- paste(candidate$cargo_path, candidate$rustc_path, sep = "|")
    if (key %in% seen) {
      next
    }
    seen <- c(seen, key)
    deduped[[length(deduped) + 1L]] <- candidate
  }
  deduped
}

select_toolchain <- function(msrv) {
  candidates <- make_toolchain_candidates()
  diagnostics <- character()

  for (candidate in candidates) {
    missing <- character()
    if (!nzchar(candidate$cargo_path)) {
      missing <- c(missing, "cargo")
    }
    if (!nzchar(candidate$rustc_path)) {
      missing <- c(missing, "rustc")
    }
    if (length(missing) > 0L) {
      diagnostics <- c(
        diagnostics,
        sprintf(
          "* %s: missing %s",
          candidate$name,
          paste(missing, collapse = " and ")
        )
      )
      next
    }

    cargo <- tryCatch(
      tool_info("cargo", candidate$cargo_path),
      error = function(e) e
    )
    if (inherits(cargo, "error")) {
      diagnostics <- c(
        diagnostics,
        sprintf("* %s: %s", candidate$name, conditionMessage(cargo))
      )
      next
    }

    rustc <- tryCatch(
      tool_info("rustc", candidate$rustc_path),
      error = function(e) e
    )
    if (inherits(rustc, "error")) {
      diagnostics <- c(
        diagnostics,
        sprintf("* %s: %s", candidate$name, conditionMessage(rustc))
      )
      next
    }

    if (utils::compareVersion(rustc$version, msrv) < 0L) {
      diagnostics <- c(
        diagnostics,
        sprintf(
          "* %s: rustc %s does not satisfy MSRV %s",
          candidate$name,
          rustc$version,
          msrv
        )
      )
      next
    }

    return(list(candidate = candidate$name, cargo = cargo, rustc = rustc))
  }

  fail(
    paste(
      c(
        "Could not find a usable Rust toolchain.",
        "Checked system PATH and personal Cargo bin locations.",
        "Details:",
        diagnostics
      ),
      collapse = "\n"
    )
  )
}

discover_rust_dependency_files <- function(rust_dir) {
  rust_dir <- normalizePath(rust_dir, winslash = "/", mustWork = TRUE)
  files <- list.files(
    rust_dir,
    recursive = TRUE,
    all.files = TRUE,
    no.. = TRUE,
    full.names = TRUE
  )
  files <- files[file.exists(files) & !dir.exists(files)]
  if (length(files) < 1L) {
    fail(sprintf("No Rust files were found under %s.", rust_dir))
  }
  files <- normalizePath(files, winslash = "/", mustWork = FALSE)
  inside <- startsWith(files, paste0(rust_dir, "/"))
  files <- files[inside]
  rel <- substring(files, nchar(rust_dir) + 2L)

  excluded_dirs <- c("target", "vendor")
  excluded_dir_pattern <- paste0(
    "(^|/)(",
    paste(excluded_dirs, collapse = "|"),
    ")/"
  )
  hidden_path_pattern <- "(^|/)[.][^/]+($|/)"
  keep <- !grepl(excluded_dir_pattern, rel) & !grepl(hidden_path_pattern, rel)
  rel <- rel[keep]
  rel <- rel[!(rel %in% c("librust.a", "roxido.txt"))]
  if (length(rel) < 1L) {
    fail(sprintf(
      "No Rust dependency files remained after filtering under %s.",
      rust_dir
    ))
  }

  deps <- file.path("rust", rel)
  sort(unique(deps))
}

render_rust_deps <- function(rust_dir, template_name) {
  deps <- c(template_name, discover_rust_dependency_files(rust_dir))
  lines <- sprintf("  %s", deps)
  if (length(lines) > 1L) {
    lines[seq_len(length(lines) - 1L)] <- paste0(
      lines[seq_len(length(lines) - 1L)],
      " \\"
    )
  }
  paste(c("RUST_DEPS = \\", lines), collapse = "\n")
}

render_makevars <- function(template_path, output_path, substitutions) {
  if (!file.exists(template_path)) {
    fail(sprintf("Could not find template: %s", template_path))
  }
  lines <- readLines(template_path, warn = FALSE)
  for (token in names(substitutions)) {
    lines <- gsub(token, substitutions[[token]], lines, fixed = TRUE)
  }
  all_text <- paste(lines, collapse = "\n")
  remaining <- unique(regmatches(all_text, gregexpr("@[A-Z_]+@", all_text))[[
    1
  ]])
  if (length(remaining) > 0L) {
    fail(sprintf(
      "Template %s has unreplaced tokens: %s",
      template_path,
      paste(remaining, collapse = ", ")
    ))
  }
  writeLines(lines, con = output_path, useBytes = TRUE)
}

make_shell_quote <- function(x) {
  escaped <- gsub("\"", "\\\\\"", x, fixed = TRUE)
  paste0("\"", escaped, "\"")
}

render_env_assignments <- function(vars) {
  if (length(vars) < 1L) {
    return("")
  }
  assignments <- sprintf(
    "%s=%s",
    names(vars),
    vapply(vars, make_shell_quote, character(1))
  )
  paste(assignments, collapse = " ")
}

render_platform_makevars <- function(
  cargo_path,
  rustc_path,
  rust_dir = file.path("src", "rust"),
  cran_vendor_mode = FALSE,
  host_triple = ""
) {
  sysname <- Sys.info()[["sysname"]]
  makevars <- file.path("src", "Makevars")
  makevars_win <- file.path("src", "Makevars.win")
  cargo_env_vars <- character()
  if (isTRUE(cran_vendor_mode)) {
    cargo_env_vars <- c(
      RUSTC = rustc_path,
      CARGO_HOME = "$(CURDIR)/$(RUST_DIR)/.cargo-home"
    )
  }
  cargo_env <- render_env_assignments(cargo_env_vars)
  cargo_sub <- make_shell_quote(cargo_path)
  unlink(c(makevars, makevars_win), force = TRUE)

  if (identical(sysname, "Windows")) {
    gnu_host <- identical(host_triple, "x86_64-pc-windows-gnu")
    if (gnu_host) {
      note(
        "GNU Windows host detected; building without --target (host = target)."
      )
      target_decl <- ""
      rust_target_lib <- "$(RUST_DIR)/target/release/librust.a"
      target_flag <- ""
      cargo_pre <- 'unset CARGO_BUILD_TARGET && export RUSTFLAGS="-C link-self-contained=yes" && '
    } else {
      note(
        "Non-GNU Windows host detected; cross-compiling with --target=x86_64-pc-windows-gnu."
      )
      target_decl <- "TARGET = x86_64-pc-windows-gnu"
      rust_target_lib <- "$(RUST_DIR)/target/$(TARGET)/release/librust.a"
      target_flag <- " --target=$(TARGET)"
      cargo_pre <- ""
    }
    render_makevars(
      file.path("src", "Makevars.win.in"),
      makevars_win,
      list(
        "@CARGO@" = cargo_sub,
        "@RUST_DEPS@" = render_rust_deps(rust_dir, "Makevars.win.in"),
        "@CARGO_ENV@" = cargo_env,
        "@TARGET_DECL@" = target_decl,
        "@RUST_TARGET_LIB@" = rust_target_lib,
        "@TARGET_FLAG@" = target_flag,
        "@CARGO_PRE@" = cargo_pre
      )
    )
    note("Rendered src/Makevars.win from template.")
  } else {
    render_makevars(
      file.path("src", "Makevars.in"),
      makevars,
      list(
        "@CARGO@" = cargo_sub,
        "@RUST_DEPS@" = render_rust_deps(rust_dir, "Makevars.in"),
        "@CARGO_ENV@" = cargo_env
      )
    )
    note("Rendered src/Makevars from template.")
  }
}

note("*** starting configure.R ***")
note(sprintf("Timestamp: %s", Sys.time()))

msrv <- read_msrv(file.path("src", "rust", "Cargo.toml"))
note(sprintf("MSRV (src/rust/Cargo.toml rust-version): %s", msrv))

toolchain <- select_toolchain(msrv)
cargo <- toolchain$cargo
rustc <- toolchain$rustc
note(sprintf("Selected Rust toolchain source: %s", toolchain$candidate))
note(sprintf("cargo path: %s", cargo$path))
note(sprintf("cargo --version: %s", cargo$line))
note(sprintf("rustc path: %s", rustc$path))
note(sprintf("rustc --version: %s", rustc$line))
note("MSRV check passed.")
rustc_vv <- tryCatch(
  system2(rustc$path, "-vV", stdout = TRUE, stderr = TRUE),
  error = function(e) "(could not run rustc -vV)"
)
note(paste(c("rustc -vV:", rustc_vv), collapse = "\n  "))

host_triple <- ""
for (line in rustc_vv) {
  m <- regexec("^\\s*host:\\s*(\\S+)", line)
  parts <- regmatches(line, m)[[1]]
  if (length(parts) >= 2L) {
    host_triple <- parts[[2]]
    break
  }
}
note(sprintf("Host triple: %s", host_triple))

cran_vendor_mode <- file.exists(file.path("src", "rust", "vendor.tar.xz"))
if (cran_vendor_mode) {
  note(
    "Detected src/rust/vendor.tar.xz; setting CARGO_HOME to a local directory."
  )
} else {
  note("No src/rust/vendor.tar.xz found; not setting CARGO_HOME.")
}

render_platform_makevars(
  cargo$path,
  rustc$path,
  cran_vendor_mode = cran_vendor_mode,
  host_triple = host_triple
)

dir.create("inst", showWarnings = FALSE, recursive = TRUE)
writeLines(cargo$version, con = file.path("inst", "cargo.log"), useBytes = TRUE)
note("Wrote inst/cargo.log.")

extract_vendor_if_present(file.path("src", "rust"))
note("*** finishing configure.R ***")

#' Start an outbound Olm session
#'
#' Use a peer's published curve25519 identity key + one of their signed
#' one-time keys to bootstrap a 1:1 ratchet. The first ciphertext from
#' this session is a pre-key message (`type = 0`).
#'
#' @param account The local Account.
#' @param peer_curve25519 Peer's curve25519 identity key (base64).
#' @param peer_otk Peer's one-time key (base64).
#' @return An Olm Session external pointer.
#' @export
mxc_olm_create_outbound <- function(account, peer_curve25519, peer_otk) {
  .Call(
    .mxc_olm_create_outbound,
    account,
    as.character(peer_curve25519),
    as.character(peer_otk)
  )
}

#' Build an inbound Olm session from a pre-key message
#'
#' Consumes the matching one-time key from the local Account. The result
#' has the new Session and the decrypted plaintext of the pre-key
#' message; subsequent messages on this session use [mxc_olm_decrypt].
#'
#' @param account The local Account (will be mutated: an OTK is consumed).
#' @param peer_curve25519 Sender's curve25519 identity key (base64).
#' @param prekey_b64 Body of the pre-key message (base64).
#' @return Named list: `session` (external pointer) and `plaintext` (raw).
#' @export
mxc_olm_create_inbound <- function(account, peer_curve25519, prekey_b64) {
  .Call(
    .mxc_olm_create_inbound,
    account,
    as.character(peer_curve25519),
    as.character(prekey_b64)
  )
}

#' Encrypt a message on an Olm session
#'
#' @param session An Olm Session.
#' @param plaintext A `raw` vector.
#' @return Named list: `type` (`0L` pre-key, `1L` normal) and `body` (base64).
#' @export
mxc_olm_encrypt <- function(session, plaintext) {
  if (!is.raw(plaintext)) {
    stop("'plaintext' must be a raw vector")
  }
  .Call(.mxc_olm_encrypt, session, plaintext)
}

#' Decrypt a message on an Olm session
#'
#' @param session An Olm Session.
#' @param type Message type: `0L` for pre-key, `1L` for normal.
#' @param body Ciphertext (base64).
#' @return A `raw` vector of plaintext bytes.
#' @export
mxc_olm_decrypt <- function(session, type, body) {
  .Call(
    .mxc_olm_decrypt,
    session,
    as.integer(type),
    as.character(body)
  )
}

#' Pickle an Olm session
#'
#' Pickle after every encrypt/decrypt to keep the ratchet state on disk.
#'
#' @param session An Olm Session.
#' @param key 32-byte raw vector.
#' @return Base64 string.
#' @export
mxc_olm_session_pickle <- function(session, key) {
  if (!is.raw(key) || length(key) != 32L) {
    stop("'key' must be a raw vector of length 32")
  }
  .Call(.mxc_olm_session_pickle, session, key)
}

#' Restore an Olm session from a pickle
#'
#' @param blob Base64 string produced by [mxc_olm_session_pickle].
#' @param key 32-byte raw vector.
#' @return An Olm Session external pointer.
#' @export
mxc_olm_session_unpickle <- function(blob, key) {
  if (!is.raw(key) || length(key) != 32L) {
    stop("'key' must be a raw vector of length 32")
  }
  .Call(.mxc_olm_session_unpickle, as.character(blob), key)
}

library(mx.crypto)

# Two parties: alice and bob. Bob publishes an OTK; alice initiates.

alice <- mxc_account_new()
bob   <- mxc_account_new()

mxc_account_generate_one_time_keys(bob, 1L)
bob_otks <- mxc_account_one_time_keys(bob)
expect_equal(length(bob_otks), 1L)

bob_idk   <- mxc_account_identity_keys(bob)
alice_idk <- mxc_account_identity_keys(alice)

# Outbound session: alice -> bob
sess_a <- mxc_olm_create_outbound(
  alice,
  peer_curve25519 = bob_idk$curve25519,
  peer_otk        = bob_otks[[1]]
)
expect_true(is(sess_a, "externalptr"))

# Alice encrypts a pre-key message
msg1_pt <- charToRaw("hello bob")
ct1 <- mxc_olm_encrypt(sess_a, msg1_pt)
expect_equal(ct1$type, 0L)
expect_true(nchar(ct1$body) > 0L)

# Bob receives the pre-key, builds inbound session
result <- mxc_olm_create_inbound(
  bob,
  peer_curve25519 = alice_idk$curve25519,
  prekey_b64      = ct1$body
)
expect_true(is(result$session, "externalptr"))
expect_true(is.raw(result$plaintext))
expect_identical(rawToChar(result$plaintext), "hello bob")
sess_b <- result$session

mxc_account_mark_published(bob)

# Bob replies; alice decrypts
msg2_pt <- charToRaw("hi alice")
ct2 <- mxc_olm_encrypt(sess_b, msg2_pt)
# Bob has not received any reply yet, so this is also pre-key
expect_true(ct2$type %in% c(0L, 1L))

dec2 <- mxc_olm_decrypt(sess_a, ct2$type, ct2$body)
expect_identical(rawToChar(dec2), "hi alice")

# Round-trip again now that both sides are warm
ct3 <- mxc_olm_encrypt(sess_a, charToRaw("third"))
dec3 <- mxc_olm_decrypt(sess_b, ct3$type, ct3$body)
expect_identical(rawToChar(dec3), "third")

# --- pickle round-trip ---------------------------------------------------

key <- as.raw(seq_len(32) - 1L)
blob <- mxc_olm_session_pickle(sess_a, key)
sess_a2 <- mxc_olm_session_unpickle(blob, key)

# Continue ratchet on the unpickled session
ct4 <- mxc_olm_encrypt(sess_a2, charToRaw("fourth"))
dec4 <- mxc_olm_decrypt(sess_b, ct4$type, ct4$body)
expect_identical(rawToChar(dec4), "fourth")

<!--
%\VignetteEngine{simplermarkdown::mdweave_to_html}
%\VignetteIndexEntry{Security audit (mx.crypto 0.2.0)}
-->
---
title: "Security audit (mx.crypto 0.2.0)"
---

# Security audit: mx.crypto 0.2.0

**Audit date:** 2026-05-13.
**Subject:** `mx.crypto` 0.1.0 (vodozemac 0.10.0 pin) → 0.2.0 with the fixes
described here.
**Trigger:** Soatok's 2026-02-17 disclosure of cryptographic issues in
vodozemac
([blog post](https://soatok.blog/2026/02/17/cryptographic-issues-in-matrixs-rust-library-vodozemac/)).
The upstream fixes for the most severe finding (non-contributory
Diffie-Hellman) shipped in vodozemac 0.10.0. The point of this audit is
not to stop there — it's to check whether **mx.crypto** itself is
positioned to take advantage of those fixes and to give callers what
they need to validate untrusted Matrix key material.

## 1. Scope and threat model

`mx.crypto` is a thin R wrapper over the Olm + Megolm primitives
in vodozemac. It sits beside `mx.api` (Matrix HTTP transport), and a
higher-level package (eventually `mx.client`) is expected to compose
the two into a real client. The threat model for the wrapper layer is
narrow but load-bearing:

- A **malicious homeserver** can return anything in `/_matrix/client/v3/keys/query`,
  `/_matrix/client/v3/keys/claim`, or `/_matrix/client/v3/sync`.
  `mx.crypto` must not let those responses corrupt session state or
  silently leak plaintext.
- A **compromised remote device** can submit signed material that
  satisfies structural checks but originates from the wrong identity.
  `mx.crypto` cannot detect this on its own — identity pinning lives
  above this layer — but `mx.crypto` must at minimum expose
  verification primitives so callers can implement TOFU /
  cross-signing without rolling their own ed25519 glue.
- A **buggy or hostile peer** can ship malformed Olm / Megolm
  ciphertext or pre-key messages. `mx.crypto` must fail closed (clear
  error, no partial state mutation) rather than returning ambiguous
  values.

Out of scope for the wrapper: cross-signing, SAS verification, and the
v2 MAC migration. Those land in a higher layer.

## 2. Dependency baseline: vodozemac 0.10.0

The Soatok post called out one **high-severity** issue and a handful of
lower-severity issues. The high-severity issue is the one that matters
most:

> Olm Diffie-Hellman accepts the identity element. When this occurs, all
> three DH computations produce zero output, resulting in a predictable
> session key.

In vodozemac 0.10.0 the fix is already in place. The relevant code in
the vendored crate (`src/types/curve25519.rs:52`) is:

```rust
/// Returns `None` if one of the keys does not show contributory behavior
/// resulting in an all-zero shared secret.
pub fn diffie_hellman(&self, their_public_key: &Curve25519PublicKey)
    -> Option<SharedSecret>
{
    let shared_secret = self.0.diffie_hellman(&their_public_key.inner);
    if shared_secret.was_contributory() { Some(shared_secret) } else { None }
}
```

`Shared3DHSecret::new` and `RemoteShared3DHSecret::new` propagate the
`None` with `?`. A dedicated regression test
(`triple_diffie_hellman_non_contributory_key`) builds an all-zero
remote one-time key and asserts that `Shared3DHSecret::new` returns
`None`. Our pinned 0.10.0 ships that test and that fix.

Strict Ed25519 verification is also the default in 0.10.0
(`src/types/ed25519.rs`), with the laxer `#[cfg(fuzzing)]` form only
compiled when the `fuzzing` Cargo feature is enabled. mx.crypto does
not enable that feature in its build.

So the *primitive* is correct. The remaining question is whether
mx.crypto correctly propagates the primitive's results.

## 3. Surface map

Before 0.2.0 mx.crypto exposed 24 functions covering:

| Group | Functions |
|---|---|
| Account | `mxc_account_new`, `mxc_account_identity_keys`, `mxc_account_sign`, `mxc_account_generate_one_time_keys`, `mxc_account_one_time_keys`, `mxc_account_mark_published`, `mxc_account_fallback_key`, `mxc_account_pickle`, `mxc_account_unpickle` |
| Olm | `mxc_olm_create_outbound`, `mxc_olm_create_inbound`, `mxc_olm_encrypt`, `mxc_olm_decrypt`, `mxc_olm_session_pickle`, `mxc_olm_session_unpickle` |
| Megolm | `mxc_megolm_outbound_new`, `mxc_megolm_outbound_info`, `mxc_megolm_encrypt`, `mxc_megolm_outbound_pickle`, `mxc_megolm_outbound_unpickle`, `mxc_megolm_inbound_new`, `mxc_megolm_decrypt`, `mxc_megolm_inbound_pickle`, `mxc_megolm_inbound_unpickle` |

What was **missing** was any way to verify signatures. `mxc_account_sign`
existed; nothing on the other side. That meant:

- `/keys/query` could return arbitrary attacker-chosen ed25519 +
  curve25519 keys for a target device and no caller could check the
  self-signature.
- `/keys/claim` could return OTKs signed under any ed25519 key the
  homeserver felt like and no caller could check.

0.2.0 adds three exports plus one internal Rust binding:

- `mxc_ed25519_verify(public_key, message, signature)` — thin
  vodozemac binding (uses `verify_strict`).
- `mxc_verify_device_keys(device_keys, expected_user_id, expected_device_id, required_algorithms = NULL)` —
  full structural + signature check for a `/keys/query` device entry.
  `required_algorithms = NULL` means "both Olm and Megolm" (i.e.
  `m.olm.v1.curve25519-aes-sha2` and `m.megolm.v1.aes-sha2`); pass
  `character(0)` to skip the algorithms check.
- `mxc_verify_one_time_key(algorithm_key_id, key_object, signing_ed25519, expected_user_id, expected_device_id)` —
  same for a `/keys/claim` entry. `algorithm_key_id` is the outer
  `"<algorithm>:<key_id>"` map key from the claim response so the
  helper can reject anything but `signed_curve25519`.

Internal-only: `mxc_curve25519_is_valid()` is a Rust binding (not
exported) used by `mxc_verify_one_time_key` to confirm the OTK's
`key` value really decodes to a 32-byte curve25519 public key, so a
signed-but-malformed key cannot pass.

The two high-level helpers are pure R; they call `mxc_ed25519_verify`
+ `mxc_curve25519_is_valid` (Rust) and `mx.api::mx_canonical_json`
(pure R) for the signed-bytes reconstruction.

## 4. Canonical JSON

`mx.crypto` does not implement canonical JSON itself — it consumes the
output of `mx.api::mx_canonical_json()`, which was audited separately
when those endpoints landed in `mx.api` 0.2.0. Key properties of that
encoder (97-assertion test suite):

- Locale-independent key sort (UTF-8 byte order, radix sort).
- Integers only (`[-(2^53)+1, (2^53)-1]`), no floats, no exponents.
- Rejects NaN, Inf, NA, NA object keys, duplicate object keys.
- Non-ASCII passes through as raw UTF-8 bytes; control chars
  `0x01-0x1F` use the spec-defined `\b \f \n \r \t` named escapes or
  `\u`-form.
- `I()` forces array encoding (matches jsonlite).

The encoder is hand-rolled — not a jsonlite wrapper — because byte
stability across implementations is the entire point. Letting another
package's default flags drift would silently break signature
verification across clients.

## 5. Finding (HIGH): `mxc_olm_create_outbound` swallowed `SessionCreationError`

vodozemac's `Account::create_outbound_session` has this signature:

```rust
pub fn create_outbound_session(
    &self,
    session_config: SessionConfig,
    identity_key: Curve25519PublicKey,
    one_time_key: Curve25519PublicKey,
) -> Result<Session, SessionCreationError>
```

The `Result::Err(SessionCreationError::NonContributoryKey)` variant is
exactly how the vodozemac fix surfaces the "remote key is all zeros"
case to the caller. mx.crypto's wrapper, however, looked like this:

```rust
fn mxc_olm_create_outbound(...) {
    let sess = acct.create_outbound_session(SessionConfig::version_1(), id_key, otk);
    RExternalPtr::encode(sess, TAG_OLM_SESSION, pc)
}
```

`RExternalPtr::encode<T>` is generic; it happily boxed a `Result<Session, _>`
into an external pointer that R hands back to the caller, with the
external-pointer tag claiming it was a `Session`. Downstream code
(`mxc_olm_encrypt`, `mxc_olm_decrypt`) then did
`ext.decode_mut::<Session>()` and used those bytes as if they were a
`Session`. That is undefined behavior in the Rust type-system sense.
In practice it "worked" on the happy path because of niche-layout
coincidences between `Result<Session, _>` and `Session`; on the error
path it produced unpredictable behavior.

### Reproducer (before the fix)

```r
library(mx.crypto)
acct <- mxc_account_new()
zero <- jsonlite::base64_enc(as.raw(rep(0, 32)))  # 32-byte zero key
sess <- mxc_olm_create_outbound(acct, zero, zero)
class(sess)
#> [1] "externalptr"               <-- no error raised
mxc_olm_encrypt(sess, charToRaw("hi"))
#> *** R hangs / aborts / returns garbage ***
```

### Severity

The vodozemac DH-zero attack itself is foiled — vodozemac correctly
refuses to produce a usable session. But mx.crypto did not surface the
refusal: a caller has no way to tell "this homeserver fed me a
malicious key" apart from "my session pointer is broken." A
defense-in-depth audit pattern (check, log, alert) cannot fire because
the failure is invisible.

There is also a memory-safety concern: the boxed `Result` is freed via
R's external-pointer finalizer with `Session`'s drop glue, which is the
wrong drop for the `Err` variant. Whether this actually corrupts the
heap depends on internal layout details we should not rely on.

### Fix

Single line: `.stop_str(...)` on the `Result`.

```rust
let sess = acct
    .create_outbound_session(SessionConfig::version_1(), id_key, otk)
    .stop_str(
        "create_outbound_session failed (e.g. non-contributory \
         Diffie-Hellman key)",
    );
RExternalPtr::encode(sess, TAG_OLM_SESSION, pc)
```

### Reproducer (after the fix)

```r
library(mx.crypto)
acct <- mxc_account_new()
zero <- jsonlite::base64_enc(as.raw(rep(0, 32)))
mxc_olm_create_outbound(acct, zero, zero)
#> Error: create_outbound_session failed (e.g. non-contributory
#> Diffie-Hellman key)
```

Covered by `inst/tinytest/test_verify.R` with `expect_error(...,
pattern = "non-contributory|create_outbound_session")`.

### Same review on the rest of the surface

Audit pass over every wrapper that consumes a vodozemac `Result`:

| Function | vodozemac call | Pre-audit | Post-audit |
|---|---|---|---|
| `mxc_olm_create_outbound` | `create_outbound_session` | **swallowed** | propagated |
| `mxc_olm_create_inbound` | `create_inbound_session` | already `.stop_str(...)` | unchanged |
| `mxc_olm_encrypt` | `Session::encrypt` | already `.stop_str(...)` | unchanged |
| `mxc_olm_decrypt` | `Session::decrypt` | already `.stop_str(...)` | unchanged |
| `mxc_megolm_inbound_new` | `InboundGroupSession::new` | returns `Self` (no error) | unchanged |
| `mxc_megolm_encrypt` | `GroupSession::encrypt` | infallible | unchanged |
| `mxc_megolm_decrypt` | `InboundGroupSession::decrypt` | already `.stop_str(...)` | unchanged |
| `*_pickle` / `*_unpickle` | `Pickle::{from,to}_encrypted` | already `.stop_str(...)` | unchanged |

Only `mxc_olm_create_outbound` was missing the propagation.

## 6. Finding (HIGH): no signature-verification primitive

Before 0.2.0 the demo at `inst/integration/e2e_demo.R` (and any other
caller) treated the homeserver's `/keys/query` and `/keys/claim`
responses as ground truth — every signature returned in those responses
went unchecked. mx.crypto's threat model documented "trust the
caller's identity-pinning layer," but the caller could not actually do
that work because mx.crypto did not expose ed25519 verification.

### Fix: three exports

The new Rust binding is a thin pass-through to vodozemac's
`Ed25519PublicKey::verify` (which itself calls `verify_strict`):

```rust
#[roxido]
fn mxc_ed25519_verify(public_key_b64: &str, message: &RObject, signature_b64: &str) {
    let pk = Ed25519PublicKey::from_base64(public_key_b64)
        .stop_str("invalid ed25519 public key (base64)");
    let sig = Ed25519Signature::from_base64(signature_b64)
        .stop_str("invalid ed25519 signature (base64 / length)");
    let msg = raw_bytes(message);
    let ok = pk.verify(msg, &sig).is_ok();
    ok.to_r(pc)
}
```

The two R helpers (`mxc_verify_device_keys`, `mxc_verify_one_time_key`)
do the Matrix-spec dance around it:

1. Strip `signatures` and `unsigned` from the object.
2. Canonicalize the remainder.
3. Verify against the ed25519 key the object claims for itself
   (`mxc_verify_device_keys`) or the ed25519 key passed in by the
   caller from a previously verified device record
   (`mxc_verify_one_time_key`).

Both helpers fail closed: every structural problem, every signer
mismatch, and every signature-bytes mismatch raises an error rather
than returning a value the caller might use by accident.

### Hostile-homeserver fixtures

`inst/tinytest/test_verify.R` builds a real signed device-keys object
and then mutates it the way a hostile homeserver would. Every variant
must raise:

| Mutation | Expected error |
|---|---|
| Wrong `expected_user_id` | "user_id mismatch" |
| Wrong `expected_device_id` | "device_id mismatch" |
| `algorithms` absent | "missing 'algorithms'" |
| `algorithms` present but missing `m.olm.v1.curve25519-aes-sha2` | "missing required entries" |
| `algorithms` empty list | "non-empty" |
| `keys` missing `curve25519:<dev>` | "missing curve25519" |
| `keys` missing `ed25519:<dev>` | "missing ed25519" |
| `signatures` block absent | "unsigned" |
| Signature only under attacker's user_id | "no signatures from" |
| Signature attached under `ed25519:OTHERDEV` | "no ed25519:ALICEDEV signature" |
| `keys.curve25519` swapped for another device's | "did not verify" |
| OTK outer key uses `curve25519:` instead of `signed_curve25519:` | "does not start with" |
| OTK outer key uses `signed_ed25519:` | "does not start with" |
| OTK outer key empty / NA | "non-empty" |
| OTK signed by attacker's ed25519 | "did not verify" |
| OTK missing `key` | "missing 'key'" |
| OTK unsigned | "unsigned" |
| OTK `key` field tampered with | "did not verify" |
| OTK `key` validly signed but not a 32-byte curve25519 key | "valid curve25519 public key" |

The full test suite is 34 assertions and lives in
`inst/tinytest/test_verify.R`. Two design choices worth flagging:

- `mxc_verify_device_keys()` validates `algorithms` against a
  caller-supplied required list (default: both Olm + Megolm). The
  default forces an explicit opt-in if you want to accept a device
  that advertises neither algorithm — there is no scenario in our
  current callers where that should silently pass.
- `mxc_verify_one_time_key()` takes the outer
  `"<algorithm>:<key_id>"` map key as a first argument so it can
  reject anything but `signed_curve25519`, and it decodes the
  `key` value through vodozemac's
  `Curve25519PublicKey::from_base64` (via the new internal
  `mxc_curve25519_is_valid` binding) so a signed-but-malformed key
  cannot pass.

### What this does NOT cover

Identity pinning. `mxc_verify_device_keys` returns the ed25519 key the
object claimed for itself, validated as self-consistent — it does not
say "this is really @alice's ed25519." That belongs to a layer above
(TOFU on first contact, then cross-signing). The helper's docstring
calls this out explicitly so callers don't assume otherwise.

## 7. DH / session error propagation (full pass)

Recap of section 5 with a wider lens. mx.crypto's R-side error handling
on the happy path is straightforward — `.Call(.mxc_*)` errors propagate
as R `simpleError`s. The risk is silent corruption: returning what
looks like a valid externalptr/value when the underlying primitive
errored. Confirmed: post-fix, every fallible vodozemac call surfaces
its error as a clean R error before any external-pointer is handed
back. No partial-state mutation paths remain.

## 8. Pickle and local state

`mxc_*_pickle()` takes a caller-supplied 32-byte raw vector. The
pickle format encrypts the serialised account/session state under that
key. Behavior:

- **Wrong key**: `mxc_*_unpickle()` errors cleanly (already covered by
  `test_account.R`).
- **Corrupted blob**: errors at decryption.
- **Right key, wrong tag (e.g. an Olm session pickle handed to
  `mxc_megolm_inbound_unpickle`)**: errors at deserialization.

Spec caveat: vodozemac's pickle uses a deterministic IV. The
implication is that reusing the same pickle key across pickles can
leak ordering information. The recommended discipline is **one pickle
key per Account / Session**, which is what callers should do anyway
for separation-of-concerns reasons. This is now stated in
`SECURITY.md`.

## 9. Other Soatok findings, mapped to our pin

| Finding | Severity | Our pin status |
|---|---|---|
| Olm DH identity element accepted | **High** | Fixed upstream + now propagated in mx.crypto (section 5) |
| Downgrade v2→v1 MAC | Low | Olm message-MAC versioning lives below mx.crypto; tracked for the v2 migration |
| ECIES CheckCode 6-bit entropy | Low | Affects QR-SAS verification, which mx.crypto does not expose |
| Drop message keys after 40 skipped | Low | Affects out-of-order delivery edge cases for buffered clients; mx.crypto inherits upstream behavior |
| Pickle deterministic IV | Low | Documented in `SECURITY.md`; per-account pickle keys recommended |
| `#[cfg(fuzzing)]` disables MAC | None | Only active under the `fuzzing` Cargo feature; mx.crypto's build does not enable it |
| Strict Ed25519 off by default | Low | vodozemac 0.10.0 uses `verify_strict` in the non-fuzzing path; our `mxc_ed25519_verify` calls that path |

## 10. API boundary with mx.api

`mx.crypto` deliberately does not depend on `mx.api` for runtime
crypto — Suggests only, used by `mxc_verify_device_keys` /
`mxc_verify_one_time_key` to canonicalize signed payloads. The bright
line:

- `mx.api` knows about HTTP and JSON; it does no signature work.
- `mx.crypto` knows about ed25519, curve25519, Olm, Megolm; it does no
  HTTP work.
- A higher layer (eventually `mx.client`) is expected to thread them
  together with identity pinning, key storage, and the broadcast /
  receive loop.

The integration demo at `inst/integration/e2e_demo.R` is a working
example of that thread. With these audit fixes in place, the demo
should also use the new verify helpers before opening Olm sessions —
that follow-up is captured in section 11.

## 11. Pending follow-ups

Not in scope for this audit; tracked for later versions.

- Wire `mxc_verify_device_keys` + `mxc_verify_one_time_key` into the
  `e2e_demo.R` broadcast loop so the demo reflects best practice.
- TOFU / cross-signing helpers (master / self / user signing keys).
- SAS verification (`m.key.verification.start` etc.).
- v2 Olm MAC migration when the Matrix spec lands.
- Track upstream vodozemac for any future disclosures and pull the
  changelog into `NEWS.md` on each bump.

## 12. Verification

Build + test results from this audit branch:

| Check | Result |
|---|---|
| `tinytest::test_package("mx.crypto")` | 22 + 18 + 11 + 34 = 85 assertions, all pass |
| `tinypkgr::check()` | 0 errors, 0 warnings (1 standard "new submission" note retained) |
| Live homeserver e2e demo | Decrypted plaintext round-trips through 4 share targets (FluffyChat verified) |
| Zero-curve25519 regression | Errors cleanly with `"non-contributory"` message |
| Tampered device_keys | Each mutation raises the expected, distinct error |

## Changelog

`mx.crypto 0.2.0`:

- **HIGH**: `mxc_olm_create_outbound` now propagates
  `SessionCreationError` (previously swallowed; non-contributory DH
  keys silently produced a corrupt session pointer).
- **HIGH**: Added `mxc_ed25519_verify`, `mxc_verify_device_keys`,
  `mxc_verify_one_time_key` so callers can validate `/keys/query` and
  `/keys/claim` responses before using them.
- `SECURITY.md` added.
- Vignette: this document.
- DESCRIPTION: bumped to 0.2.0; added `mx.api (>= 0.2.0)` and
  `simplermarkdown` to Suggests; `VignetteBuilder: simplermarkdown`.

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_account_mark_published}
\alias{mxc_account_mark_published}
\title{Mark current one-time keys as published}
\usage{
mxc_account_mark_published(account)
}
\arguments{
\item{account}{An Account.}
}
\value{
Invisible NULL; mutates `account`.
}
\description{
Call after the homeserver has accepted a `/keys/upload` containing
the keys returned by [mxc_account_one_time_keys]. Future calls to
that function will not re-include them.
}

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_olm_session_pickle}
\alias{mxc_olm_session_pickle}
\title{Pickle an Olm session}
\usage{
mxc_olm_session_pickle(session, key)
}
\arguments{
\item{session}{An Olm Session.}

\item{key}{32-byte raw vector.}
}
\value{
Base64 string.
}
\description{
Pickle after every encrypt/decrypt to keep the ratchet state on disk.
}

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_olm_encrypt}
\alias{mxc_olm_encrypt}
\title{Encrypt a message on an Olm session}
\usage{
mxc_olm_encrypt(session, plaintext)
}
\arguments{
\item{session}{An Olm Session.}

\item{plaintext}{A `raw` vector.}
}
\value{
Named list: `type` (`0L` pre-key, `1L` normal) and `body` (base64).
}
\description{
Encrypt a message on an Olm session
}

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_megolm_encrypt}
\alias{mxc_megolm_encrypt}
\title{Encrypt a room message}
\usage{
mxc_megolm_encrypt(gs, plaintext)
}
\arguments{
\item{gs}{An outbound GroupSession.}

\item{plaintext}{A `raw` vector. The caller is responsible for
producing the canonical JSON event body.}
}
\value{
Base64 ciphertext (the `ciphertext` field of `m.room.encrypted`).
}
\description{
Encrypt a room message
}

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_verify_one_time_key}
\alias{mxc_verify_one_time_key}
\title{Verify a signed one-time / fallback key}
\usage{
mxc_verify_one_time_key(algorithm_key_id, key_object, signing_ed25519,
                        expected_user_id, expected_device_id)
}
\arguments{
\item{algorithm_key_id}{Character. The outer map key from the
\code{one_time_keys} or \code{fallback_keys} response, e.g.
\code{"signed_curve25519:AAAAAAAAAAA"}. The algorithm prefix
must be \code{signed_curve25519}.}

\item{key_object}{Named list. The signed key object (must contain
\code{key} and \code{signatures}).}

\item{signing_ed25519}{Character. The base64 ed25519 public key
that should have signed this OTK (i.e. the ed25519 returned by
\code{mxc_verify_device_keys} for the same device).}

\item{expected_user_id}{Character. Matrix user id the OTK is
supposed to come from.}

\item{expected_device_id}{Character. Matrix device id.}
}
\value{
Character. The verified curve25519 OTK (base64), ready to
  feed into \code{mxc_olm_create_outbound}.
}
\description{
Validates a single signed_curve25519 object returned by
\code{/_matrix/client/v3/keys/claim} (or the \code{fallback_keys}
block of \code{/keys/upload}). The signing ed25519 key must come
from a previously verified \code{device_keys} object —
\emph{this function does not look it up for you}, because doing so
would silently re-trust whatever the homeserver hands back.
The caller must pass the outer map key (\code{"<algorithm>:<key_id>"})
alongside the inner key object so the helper can reject anything but
\code{signed_curve25519}. The helper also confirms the \code{key}
value decodes to a 32-byte curve25519 public key; a signature over
garbage bytes is not a useful "verified" result.
}
\examples{
\dontrun{
cl <- mx.api::mx_keys_claim(s, list(
  "@alice:server" = list("ALICEDEV" = "signed_curve25519")
))
entry <- cl$one_time_keys[["@alice:server"]][["ALICEDEV"]]
algo_kid <- names(entry)[[1]]   # e.g. "signed_curve25519:AAAA"
otk <- mxc_verify_one_time_key(
  algo_kid, entry[[1]], alice_ed, "@alice:server", "ALICEDEV"
)
}
}

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_account_generate_one_time_keys}
\alias{mxc_account_generate_one_time_keys}
\title{Generate one-time keys}
\usage{
mxc_account_generate_one_time_keys(account, n)
}
\arguments{
\item{account}{An Account.}

\item{n}{Number of OTKs to generate.}
}
\value{
Invisible NULL; mutates `account` in place.
}
\description{
Adds `n` curve25519 one-time keys to the Account's pool. Call
[mxc_account_one_time_keys] to read them out for upload, then
[mxc_account_mark_published] once the homeserver has accepted them.
}
\examples{
acct <- mxc_account_new()
mxc_account_generate_one_time_keys(acct, 5L)
}

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_account_fallback_key}
\alias{mxc_account_fallback_key}
\title{Generate and return a fallback key}
\usage{
mxc_account_fallback_key(account)
}
\arguments{
\item{account}{An Account.}
}
\value{
Named list with `key_id` and `curve25519` (both base64).
}
\description{
Generates a fallback curve25519 key, which the homeserver hands out
when an OTK pool is exhausted. Returns the freshly generated key.
Calling it again rotates the previous fallback.
}

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_megolm_inbound_new}
\alias{mxc_megolm_inbound_new}
\title{Build an inbound Megolm session from a shared session_key}
\usage{
mxc_megolm_inbound_new(session_key)
}
\arguments{
\item{session_key}{Base64 session_key from `m.room_key`.}
}
\value{
An InboundGroupSession external pointer.
}
\description{
Constructs the receiver-side ratchet using the `session_key` that
was delivered (over Olm) in an `m.room_key` event. The receiver
stores the resulting object indexed by `(sender_curve25519,
session_id)` and re-uses it to decrypt incoming messages on that
session.
}

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_megolm_decrypt}
\alias{mxc_megolm_decrypt}
\title{Decrypt a room message}
\usage{
mxc_megolm_decrypt(igs, ciphertext_b64)
}
\arguments{
\item{igs}{An InboundGroupSession.}

\item{ciphertext_b64}{Base64 ciphertext (the `ciphertext` field of
the `m.room.encrypted` event).}
}
\value{
Named list: `plaintext` (raw) and `message_index` (integer).
  Use `message_index` to dedupe replays.
}
\description{
Decrypt a room message
}

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_verify_device_keys}
\alias{mxc_verify_device_keys}
\title{Verify a Matrix device-keys object}
\usage{
mxc_verify_device_keys(device_keys, expected_user_id, expected_device_id,
                       required_algorithms = NULL)
}
\arguments{
\item{device_keys}{Named list. The \code{device_keys} object,
exactly as parsed from a homeserver response (no signatures
block stripped).}

\item{expected_user_id}{Character. Matrix user id the device is
supposed to belong to.}

\item{expected_device_id}{Character. The device id we're verifying.}

\item{required_algorithms}{Character vector or NULL. Algorithm
identifiers that must appear in \code{device_keys$algorithms}
for the device to be accepted. \code{NULL} (the default) means
"both Olm and Megolm" — i.e. \code{m.olm.v1.curve25519-aes-sha2}
and \code{m.megolm.v1.aes-sha2}. Pass \code{character(0)} to
skip the check, but be aware that the audit plan requires it.}
}
\value{
Named list with the verified \code{curve25519} and
  \code{ed25519} public keys (both base64). Use these as the
  inputs for Olm session establishment.
}
\description{
Validates a single \code{device_keys} object as returned by
\code{/_matrix/client/v3/keys/query}: checks that the structural
fields (user_id, device_id, algorithms, keys, signatures) are
present and match the expected identity, then verifies the
ed25519 self-signature over the canonical-JSON byte sequence
(with \code{signatures} and \code{unsigned} stripped).
Any of the following raises an error rather than returning
\code{FALSE}: missing field, wrong \code{user_id} or
\code{device_id}, missing curve25519 / ed25519 key, missing
signatures block, signature under the wrong user or device,
invalid signature, malformed base64.
This function deliberately accepts \emph{any} ed25519 key the
object claims for itself; it does not pin against a previously
trusted key. Identity pinning is the caller's responsibility
(typically TOFU + cross-signing).
}
\examples{
\dontrun{
q <- mx.api::mx_keys_query(s, list("@alice:server" = "ALICEDEV"))
dk <- q$device_keys[["@alice:server"]][["ALICEDEV"]]
keys <- mxc_verify_device_keys(dk, "@alice:server", "ALICEDEV")
keys$curve25519
}
}

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_account_one_time_keys}
\alias{mxc_account_one_time_keys}
\title{Read pending one-time keys}
\usage{
mxc_account_one_time_keys(account)
}
\arguments{
\item{account}{An Account.}
}
\value{
Named list mapping `key_id` to `curve25519_pub` (both
  unpadded base64 strings).
}
\description{
Returns OTKs that have been generated but not yet marked as published.
Each value is a curve25519 public key; the caller should sign each
with [mxc_account_sign] and upload as `signed_curve25519:<key_id>`.
}
\examples{
acct <- mxc_account_new()
mxc_account_generate_one_time_keys(acct, 5L)
otks <- mxc_account_one_time_keys(acct)
}

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_megolm_outbound_info}
\alias{mxc_megolm_outbound_info}
\title{Inspect an outbound group session}
\usage{
mxc_megolm_outbound_info(gs)
}
\arguments{
\item{gs}{An outbound GroupSession.}
}
\value{
Named list: `session_id` (str), `session_key` (str),
  `message_index` (int).
}
\description{
Returns the `session_id`, current `session_key` (the value to ship
via `m.room_key` to recipient devices), and `message_index`.
}

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_account_unpickle}
\alias{mxc_account_unpickle}
\title{Restore an Account from an encrypted pickle}
\usage{
mxc_account_unpickle(blob, key)
}
\arguments{
\item{blob}{A base64 string produced by [mxc_account_pickle].}

\item{key}{The 32-byte key the pickle was encrypted under.}
}
\value{
An Account external pointer.
}
\description{
Restore an Account from an encrypted pickle
}

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_megolm_outbound_new}
\alias{mxc_megolm_outbound_new}
\title{Create an outbound Megolm group session}
\usage{
mxc_megolm_outbound_new()
}
\value{
An outbound GroupSession external pointer.
}
\description{
The sender side of a Megolm ratchet. Use [mxc_megolm_outbound_info]
to read out the `session_key` that must be shared (over Olm) with
each recipient device, and [mxc_megolm_encrypt] to encrypt room
messages. Rotate (create a new one) on membership changes or after
your chosen number-of-messages / time threshold.
}

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_account_sign}
\alias{mxc_account_sign}
\title{Sign canonical JSON with the Account's ed25519 key}
\usage{
mxc_account_sign(account, canonical_json)
}
\arguments{
\item{account}{An Account.}

\item{canonical_json}{A character string containing canonical JSON.}
}
\value{
Unpadded base64 ed25519 signature.
}
\description{
The caller must canonicalise the JSON per the Matrix spec
(UTF-8, sorted keys, no whitespace, no insignificant data) before
passing it in. mx.crypto does not canonicalise; mx.api will.
}
\examples{
acct <- mxc_account_new()
sig <- mxc_account_sign(acct, '{"hello":"world"}')
}

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_megolm_outbound_pickle}
\alias{mxc_megolm_outbound_pickle}
\title{Pickle an outbound group session}
\usage{
mxc_megolm_outbound_pickle(gs, key)
}
\arguments{
\item{gs}{An outbound GroupSession.}

\item{key}{32-byte raw vector.}
}
\value{
Base64 string.
}
\description{
Pickle an outbound group session
}

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_olm_create_inbound}
\alias{mxc_olm_create_inbound}
\title{Build an inbound Olm session from a pre-key message}
\usage{
mxc_olm_create_inbound(account, peer_curve25519, prekey_b64)
}
\arguments{
\item{account}{The local Account (will be mutated: an OTK is consumed).}

\item{peer_curve25519}{Sender's curve25519 identity key (base64).}

\item{prekey_b64}{Body of the pre-key message (base64).}
}
\value{
Named list: `session` (external pointer) and `plaintext` (raw).
}
\description{
Consumes the matching one-time key from the local Account. The result
has the new Session and the decrypted plaintext of the pre-key
message; subsequent messages on this session use [mxc_olm_decrypt].
}

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_ed25519_verify}
\alias{mxc_ed25519_verify}
\title{Verify an Ed25519 signature}
\usage{
mxc_ed25519_verify(public_key, message, signature)
}
\arguments{
\item{public_key}{Character. Unpadded base64 ed25519 public key
(the same shape \code{mxc_account_identity_keys} returns).}

\item{message}{A \code{raw} vector. The byte sequence the signer
ran ed25519 over; typically the output of
\code{mx.api::mx_canonical_json()}.}

\item{signature}{Character. Unpadded base64 ed25519 signature
(86 chars).}
}
\value{
Single logical: \code{TRUE} for a valid signature,
  \code{FALSE} otherwise.
}
\description{
Thin wrapper over vodozemac's strict Ed25519 verifier
(\code{verify_strict}). Returns \code{TRUE} when the signature is
valid and \code{FALSE} when it isn't; raises an error if any of
\code{public_key}, \code{message}, or \code{signature} is malformed
(bad base64, wrong length, etc.).
}
\examples{
acct <- mxc_account_new()
ids <- mxc_account_identity_keys(acct)
sig <- mxc_account_sign(acct, "{\"hello\":\"world\"}")
mxc_ed25519_verify(ids$ed25519, charToRaw("{\"hello\":\"world\"}"), sig)
}

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_olm_create_outbound}
\alias{mxc_olm_create_outbound}
\title{Start an outbound Olm session}
\usage{
mxc_olm_create_outbound(account, peer_curve25519, peer_otk)
}
\arguments{
\item{account}{The local Account.}

\item{peer_curve25519}{Peer's curve25519 identity key (base64).}

\item{peer_otk}{Peer's one-time key (base64).}
}
\value{
An Olm Session external pointer.
}
\description{
Use a peer's published curve25519 identity key + one of their signed
one-time keys to bootstrap a 1:1 ratchet. The first ciphertext from
this session is a pre-key message (`type = 0`).
}

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_olm_decrypt}
\alias{mxc_olm_decrypt}
\title{Decrypt a message on an Olm session}
\usage{
mxc_olm_decrypt(session, type, body)
}
\arguments{
\item{session}{An Olm Session.}

\item{type}{Message type: `0L` for pre-key, `1L` for normal.}

\item{body}{Ciphertext (base64).}
}
\value{
A `raw` vector of plaintext bytes.
}
\description{
Decrypt a message on an Olm session
}

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_megolm_inbound_unpickle}
\alias{mxc_megolm_inbound_unpickle}
\title{Restore an inbound group session from a pickle}
\usage{
mxc_megolm_inbound_unpickle(blob, key)
}
\arguments{
\item{blob}{Base64 string.}

\item{key}{32-byte raw vector.}
}
\value{
An InboundGroupSession external pointer.
}
\description{
Restore an inbound group session from a pickle
}

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_account_identity_keys}
\alias{mxc_account_identity_keys}
\title{Public identity keys for an Account}
\usage{
mxc_account_identity_keys(account)
}
\arguments{
\item{account}{An Account from [mxc_account_new] or [mxc_account_unpickle].}
}
\value{
A named list with `curve25519` and `ed25519` strings.
}
\description{
Returns the device's two public keys: curve25519 (Olm sender / identity
key, used to start sessions) and ed25519 (signing / fingerprint key).
Both are unpadded base64 strings, ready for `/keys/upload`.
}
\examples{
k <- mxc_account_identity_keys(mxc_account_new())
k$curve25519
}

% tinyrox says don't edit this manually, but it can't stop you!
\docType{package}
\name{mx.crypto-package}
\alias{mx.crypto}
\alias{mx.crypto-package}
\title{mx.crypto: Matrix End-to-End Encryption Primitives}
\description{
Olm and Megolm ratchet primitives for the Matrix protocol, wrapping
the `vodozemac` Rust crate. Pairs with `mx.api`, which handles HTTP
transport. mx.crypto is crypto only: no network, no canonical-JSON,
no `m.room_key_request`, no cross-signing or SAS in 0.1.0.
}
\author{
\strong{Maintainer}: Troy Hernandez \email{troy@cornball.ai}
}

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_olm_session_unpickle}
\alias{mxc_olm_session_unpickle}
\title{Restore an Olm session from a pickle}
\usage{
mxc_olm_session_unpickle(blob, key)
}
\arguments{
\item{blob}{Base64 string produced by [mxc_olm_session_pickle].}

\item{key}{32-byte raw vector.}
}
\value{
An Olm Session external pointer.
}
\description{
Restore an Olm session from a pickle
}

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_account_pickle}
\alias{mxc_account_pickle}
\title{Pickle an Account to an encrypted blob}
\usage{
mxc_account_pickle(account, key)
}
\arguments{
\item{account}{An Account.}

\item{key}{A `raw` vector of length 32. The caller is responsible for
key derivation; use a KDF on a passphrase, do not pass a passphrase.}
}
\value{
Base64 string containing the encrypted pickle.
}
\description{
Serialises the Account's state and encrypts it under a 32-byte key.
Restore with [mxc_account_unpickle]. Pickle after every state change
that you want to survive a restart (OTK generation, fallback key
rotation, mark-published).
}

% tinyrox says don't edit this manually, but it can't stop you!
\name{mxc_account_new}
\alias{mxc_account_new}
\title{Create a new Olm Account}
\usage{
mxc_account_new()
}
\value{
An external pointer to a vodozemac Account.
}
\description{
Creates a fresh device identity. Holds long-lived curve25519 / ed25519
identity keys and a one-time-key pool. The returned object is an
external pointer; persist it with [mxc_account_pickle].
}
\examples{
acct <- mxc_account_new()
mxc_account_identity_keys(acct)
}
